@@ -14,7 +14,11 @@ import pydantic as pd
 import pydantic.v1 as pd_v1
 
 from .. import error_messages
-from ..cloud.flow360_requests import MoveCaseItem, MoveToFolderRequest
+from ..cloud.flow360_requests import (
+    MoveCaseItem,
+    MoveToFolderRequest,
+    RenameAssetRequestV2,
+)
 from ..cloud.rest_api import RestApi
 from ..cloud.s3_utils import CloudFileNotFoundError
 from ..exceptions import Flow360RuntimeError, Flow360ValidationError, Flow360ValueError
@@ -178,7 +182,6 @@ class CaseMetaV2(AssetMetaBaseModelV2):
     """
 
     id: str = pd.Field(alias="caseId")
-    case_mesh_id: str = pd.Field(alias="caseMeshId")
     status: Flow360Status = pd.Field()
 
     def to_case(self) -> Case:
@@ -454,12 +457,14 @@ class Case(CaseBase, Flow360Resource):
             params_as_dict = self._parse_json_from_cloud("simulation.json")
         except CloudFileNotFoundError as err:
             raise Flow360ValueError(
-                "Simulation params not found for this case. It is likely it was created with old interface"
+                "Simulation params not found for this case. It is likely it was created with old interface."
+                f" Original error: {str(err)}"
             ) from err
 
         # if the params come from GUI, it can contain data that is not conformal with SimulationParams thus cleaning
         param, errors, _ = services.validate_model(
             params_as_dict=params_as_dict,
+            validated_by=services.ValidationCalledBy.LOCAL,
             root_item_type=None,
             validation_level=None,
         )
@@ -534,12 +539,35 @@ class Case(CaseBase, Flow360Resource):
         raise Flow360RuntimeError("Case does not have parent case.")
 
     @property
-    def info(self) -> CaseMeta:
+    def info(self) -> CaseMetaV2:
         """
         returns metadata info for case
         """
         return super().info
 
+    @property
+    def info_v2(self) -> CaseMetaV2:
+        """
+        returns metadata v2 info for case
+        """
+        return self._web_api_v2.info
+
+    @property
+    def project_id(self) -> Optional[str]:
+        """Returns the project id of the case if case was run with V2 interface."""
+        if isinstance(self.info, CaseMeta):
+            return self.info.projectId
+        if isinstance(self.info, CaseMetaV2):
+            return self.info.project_id
+        raise ValueError("Case info is not of type CaseMeta or CaseMetaV2")
+
+    @property
+    def tags(self) -> List[str]:
+        """
+        get case tags
+        """
+        return self._web_api_v2.info.tags
+
     @property
     def volume_mesh(self) -> "VolumeMeshV2":
         """
@@ -657,6 +685,19 @@ class Case(CaseBase, Flow360Resource):
         )
         return self
 
+    def rename(self, new_name: str):
+        """
+        Rename the current case.
+
+        Parameters
+        ----------
+        new_name : str
+            The new name for the case.
+        """
+        RestApi(CaseInterfaceV2.endpoint).patch(
+            RenameAssetRequestV2(name=new_name).dict(), method=self.id
+        )
+
     @classmethod
     def _interface(cls):
         return CaseInterface
@@ -853,7 +894,7 @@ class CaseResultsModel(pd.BaseModel):
         if not isinstance(self.case, Case):
             raise TypeError("case must be of type Case")
 
-        for field_name in self.model_fields:
+        for field_name in self.__class__.model_fields:  # pylint:disable = not-an-iterable
             value = getattr(self, field_name)
             if isinstance(value, ResultBaseModel):
                 # pylint: disable=protected-access,no-member
@@ -891,7 +932,7 @@ class CaseResultsModel(pd.BaseModel):
             "user_defined_dynamics": self.case.has_user_defined_dynamics,
         }
 
-        for field_name in self.model_fields:
+        for field_name in self.__class__.model_fields:  # pylint:disable = not-an-iterable
             value = getattr(self, field_name)
             if isinstance(value, ResultBaseModel):
                 function = has_function_map.get(field_name, lambda: True)
@@ -1066,7 +1107,7 @@ class CaseResultsModel(pd.BaseModel):
         keep_remote_structure : bool, optional
             When true, remote folder structure is assumed to be preserved, otherwise flat structure, by default False
         """
-        for field_name in self.model_fields:
+        for field_name in self.__class__.model_fields:  # pylint:disable = not-an-iterable
             value = getattr(self, field_name)
             if isinstance(value, ResultBaseModel):
                 if keep_remote_structure is True:
