@@ -7,7 +7,7 @@ from __future__ import annotations
 import os
 import threading
 from enum import Enum
-from typing import Any, List, Literal, Union
+from typing import Any, List, Literal, Optional, Union
 
 import pydantic as pd
 
@@ -19,18 +19,22 @@ from flow360.cloud.flow360_requests import (
 from flow360.cloud.heartbeat import post_upload_heartbeat
 from flow360.cloud.rest_api import RestApi
 from flow360.component.interfaces import GeometryInterface
-from flow360.component.project_utils import GeometryFiles
 from flow360.component.resource_base import (
     AssetMetaBaseModelV2,
     Flow360Resource,
     ResourceDraft,
 )
 from flow360.component.simulation.entity_info import GeometryEntityInfo
-from flow360.component.simulation.framework.entity_registry import EntityRegistry
-from flow360.component.simulation.primitives import Edge, Surface
+from flow360.component.simulation.folder import Folder
+from flow360.component.simulation.primitives import Edge, GeometryBodyGroup, Surface
+from flow360.component.simulation.unit_system import LengthType
 from flow360.component.simulation.utils import model_attribute_unlock
 from flow360.component.simulation.web.asset_base import AssetBase
-from flow360.component.utils import shared_account_confirm_proceed
+from flow360.component.utils import (
+    GeometryFiles,
+    MeshNameParser,
+    shared_account_confirm_proceed,
+)
 from flow360.exceptions import Flow360FileError, Flow360ValueError
 from flow360.log import log
 
@@ -88,12 +92,14 @@ class GeometryDraft(ResourceDraft):
         solver_version: str = None,
         length_unit: LengthUnitType = "m",
         tags: List[str] = None,
+        folder: Optional[Folder] = None,
     ):
         self._file_names = file_names
         self.project_name = project_name
         self.tags = tags if tags is not None else []
         self.length_unit = length_unit
         self.solver_version = solver_version
+        self.folder = folder
         self._validate()
         ResourceDraft.__init__(self)
 
@@ -161,7 +167,17 @@ class GeometryDraft(ResourceDraft):
 
         if not shared_account_confirm_proceed():
             raise Flow360ValueError("User aborted resource submit.")
-        # The first geometry is assumed to be the main one.
+        mapbc_files = []
+        for file_path in self.file_names:
+            mesh_parser = MeshNameParser(file_path)
+            if mesh_parser.is_ugrid() and os.path.isfile(
+                mesh_parser.get_associated_mapbc_filename()
+            ):
+                file_name_mapbc = os.path.basename(mesh_parser.get_associated_mapbc_filename())
+                mapbc_files.append(file_name_mapbc)
+
+        # Files with 'main' type are treated as MASTER_FILES and are processed after uploading
+        # 'dependency' type files are uploaded only but not processed.
         req = NewGeometryRequest(
             name=self.project_name,
             solver_version=self.solver_version,
@@ -169,13 +185,11 @@ class GeometryDraft(ResourceDraft):
             files=[
                 GeometryFileMeta(
                     name=os.path.basename(file_path),
-                    type="main" if item_index == 0 else "dependency",
+                    type="main",
                 )
-                for item_index, file_path in enumerate(self.file_names)
+                for file_path in self.file_names + mapbc_files
             ],
-            # pylint: disable=fixme
-            # TODO: remove hardcoding
-            parent_folder_id="ROOT.FLOW360",
+            parent_folder_id=self.folder.id if self.folder else "ROOT.FLOW360",
             length_unit=self.length_unit,
             description=description,
         )
@@ -190,7 +204,7 @@ class GeometryDraft(ResourceDraft):
         # Keep posting the heartbeat to keep server patient about uploading.
         heartbeat_thread = threading.Thread(target=post_upload_heartbeat, args=(heartbeat_info,))
         heartbeat_thread.start()
-        for file_path in self.file_names:
+        for file_path in self.file_names + mapbc_files:
             geometry._webapi._upload_file(
                 remote_file_name=os.path.basename(file_path),
                 file_name=file_path,
@@ -229,8 +243,7 @@ class Geometry(AssetBase):
 
     @face_group_tag.setter
     def face_group_tag(self, new_value: str):
-        with model_attribute_unlock(self._entity_info, "face_group_tag"):
-            self._entity_info.face_group_tag = new_value
+        raise SyntaxError("Cannot set face_group_tag, use group_faces_by_tag() instead.")
 
     @property
     def edge_group_tag(self):
@@ -239,30 +252,40 @@ class Geometry(AssetBase):
 
     @edge_group_tag.setter
     def edge_group_tag(self, new_value: str):
-        with model_attribute_unlock(self._entity_info, "edge_group_tag"):
-            self._entity_info.edge_group_tag = new_value
+        raise SyntaxError("Cannot set edge_group_tag, use group_edges_by_tag() instead.")
+
+    @property
+    def body_group_tag(self):
+        "getter for body_group_tag"
+        return self._entity_info.body_group_tag
+
+    @body_group_tag.setter
+    def body_group_tag(self, new_value: str):
+        raise SyntaxError("Cannot set body_group_tag, use group_bodies_by_tag() instead.")
+
+    def get_default_settings(self, simulation_dict: dict):
+        """Get the default geometry settings from the simulation dict"""
+
+        def _get_default_geometry_accuracy(simulation_dict: dict) -> LengthType.Positive:
+            """Get the default geometry accuracy from the simulation json"""
+            if simulation_dict.get("meshing") is None:
+                return None
+            if simulation_dict["meshing"].get("defaults") is None:
+                return None
+            if simulation_dict["meshing"]["defaults"].get("geometry_accuracy") is None:
+                return None
+            # pylint: disable=no-member
+            return LengthType.validate(simulation_dict["meshing"]["defaults"]["geometry_accuracy"])
+
+        self.default_settings["geometry_accuracy"] = _get_default_geometry_accuracy(
+            simulation_dict=simulation_dict
+        )
 
     @classmethod
     # pylint: disable=redefined-builtin
-    def from_cloud(cls, id: str, **kwargs):
+    def from_cloud(cls, id: str, **kwargs) -> Geometry:
         """Create asset with the given ID"""
         asset_obj = super().from_cloud(id, **kwargs)
-        # get the face tag and edge tag used.
-        # pylint: disable=no-member
-        if asset_obj.face_group_tag is None:
-            # This may happen if users submit the Geometry but did not do anything else.
-            # Then they load back the geometry which will then have no info on grouping.
-            log.warning(
-                "Could not find face grouping info in the draft's simulation settings. "
-                "Please remember to group them if relevant features are used."
-            )
-
-        if asset_obj.edge_group_tag is None:
-            log.warning(
-                "Could not find edge grouping info in the draft's simulation settings. "
-                "Please remember to group them if relevant features are used."
-            )
-
         return asset_obj
 
     @classmethod
@@ -274,9 +297,12 @@ class Geometry(AssetBase):
         solver_version: str = None,
         length_unit: LengthUnitType = "m",
         tags: List[str] = None,
+        folder: Optional[Folder] = None,
     ) -> GeometryDraft:
         # For type hint only but proper fix is to fully abstract the Draft class too.
-        return super().from_file(file_names, project_name, solver_version, length_unit, tags)
+        return super().from_file(
+            file_names, project_name, solver_version, length_unit, tags, folder=folder
+        )
 
     def show_available_groupings(self, verbose_mode: bool = False):
         """Display all the possible groupings for faces and edges"""
@@ -290,6 +316,11 @@ class Geometry(AssetBase):
             ignored_attribute_tags=["__all__", "edgeId"],
             show_ids_in_each_group=verbose_mode,
         )
+        self._show_available_entity_groups(
+            "bodies",
+            ignored_attribute_tags=["__all__", "bodyId"],
+            show_ids_in_each_group=verbose_mode,
+        )
 
     @classmethod
     def from_local_storage(
@@ -316,7 +347,7 @@ class Geometry(AssetBase):
 
     def _show_available_entity_groups(
         self,
-        entity_type_name: Literal["faces", "edges"],
+        entity_type_name: Literal["faces", "edges", "bodies"],
         ignored_attribute_tags: list = None,
         show_ids_in_each_group: bool = False,
     ) -> None:
@@ -324,18 +355,22 @@ class Geometry(AssetBase):
         Display all the grouping info for the given entity type
         """
 
-        if entity_type_name not in ["faces", "edges"]:
+        if entity_type_name not in ["faces", "edges", "bodies"]:
             raise Flow360ValueError(
-                f"entity_type_name: {entity_type_name} is invalid. Valid options are: ['faces', 'edges']"
+                f"entity_type_name: {entity_type_name} is invalid. Valid options are: ['faces', 'edges', 'bodies']"
             )
 
         # pylint: disable=no-member
         if entity_type_name == "faces":
             attribute_names = self._entity_info.face_attribute_names
             grouped_items = self._entity_info.grouped_faces
-        else:
+        elif entity_type_name == "edges":
             attribute_names = self._entity_info.edge_attribute_names
             grouped_items = self._entity_info.grouped_edges
+        else:
+            attribute_names = self._entity_info.body_attribute_names
+            grouped_items = self._entity_info.grouped_bodies
+
         log.info(f" >> Available attribute tags for grouping **{entity_type_name}**:")
         for tag_index, attribute_tag in enumerate(attribute_names):
             if ignored_attribute_tags is not None and attribute_tag in ignored_attribute_tags:
@@ -344,65 +379,172 @@ class Geometry(AssetBase):
                 f"    >> Tag '{tag_index}': {attribute_tag}. Grouping with this tag results in:"
             )
             for index, entity in enumerate(grouped_items[tag_index]):
-                log.info(f"        >> Boundary {index}: {entity.name}")
+                log.info(f"        >> [{index}]: {entity.name}")
                 if show_ids_in_each_group is True:
                     log.info(f"           IDs: {entity.private_attribute_sub_components}")
 
-    def _group_entity_by_tag(
-        self, entity_type_name: Literal["face", "edge"], tag_name: str
-    ) -> None:
-        if hasattr(self, "internal_registry") is False or self.internal_registry is None:
-            self.internal_registry = EntityRegistry()
-
-        found_existing_grouping = (
-            self.face_group_tag is not None
-            if entity_type_name == "face"
-            else self.edge_group_tag is not None
-        )
-        if found_existing_grouping is True:
-            log.warning(
-                f"Grouping already exists for {entity_type_name}. Resetting the grouping and regroup with {tag_name}."
-            )
-            self._reset_grouping(entity_type_name)
-
-        self.internal_registry = self._entity_info.group_in_registry(
-            entity_type_name, attribute_name=tag_name, registry=self.internal_registry
-        )
-        if entity_type_name == "face":
-            self.face_group_tag = tag_name
-        else:
-            self.edge_group_tag = tag_name
-
     def group_faces_by_tag(self, tag_name: str) -> None:
         """
         Group faces by tag name
         """
-        self._group_entity_by_tag("face", tag_name)
+        # pylint: disable=protected-access,no-member
+        self.internal_registry = self._entity_info._group_entity_by_tag(
+            "face", tag_name, self.internal_registry
+        )
 
     def group_edges_by_tag(self, tag_name: str) -> None:
         """
         Group edges by tag name
         """
-        self._group_entity_by_tag("edge", tag_name)
+        # pylint: disable=protected-access,no-member
+        self.internal_registry = self._entity_info._group_entity_by_tag(
+            "edge", tag_name, self.internal_registry
+        )
 
-    def _reset_grouping(self, entity_type_name: Literal["face", "edge"]) -> None:
-        if entity_type_name == "face":
-            self.internal_registry.clear(Surface)
-        else:
-            self.internal_registry.clear(Edge)
-
-        if entity_type_name == "face":
-            self.face_group_tag = None
-        else:
-            self.edge_group_tag = None
+    def group_bodies_by_tag(self, tag_name: str) -> None:
+        """
+        Group bodies by tag name
+        """
+        # pylint: disable=protected-access,no-member
+        self.internal_registry = self._entity_info._group_entity_by_tag(
+            "body", tag_name, self.internal_registry
+        )
 
     def reset_face_grouping(self) -> None:
         """Reset the face grouping"""
-        self._reset_grouping("face")
+        # pylint: disable=protected-access,no-member
+        self.internal_registry = self._entity_info._reset_grouping("face", self.internal_registry)
 
     def reset_edge_grouping(self) -> None:
         """Reset the edge grouping"""
-        self._reset_grouping("edge")
+        # pylint: disable=protected-access,no-member
+        self.internal_registry = self._entity_info._reset_grouping("edge", self.internal_registry)
+
+    def reset_body_grouping(self) -> None:
+        """Reset the body grouping"""
+        # pylint: disable=protected-access,no-member
+        self.internal_registry = self._entity_info._reset_grouping("body", self.internal_registry)
+
+    def _rename_entity(
+        self,
+        entity_type_name: Literal["face", "edge", "body"],
+        current_name_pattern: str,
+        new_name_prefix: str,
+    ):
+        """
+        Rename the entity
+
+        Parameters
+        ----------
+        entity_type_name : Literal["face", "edge", "body"]
+            The type of entity that needs renaming
+
+        current_name_pattern:
+            The current name of a single entity or the name pattern of the entities
+
+        new_name_prefix:
+            The new name of a single entity or the new name prefix of the entities
+
+        """
+
+        # pylint: disable=too-many-boolean-expressions
+        if (
+            (entity_type_name == "face" and not self.face_group_tag)
+            or (entity_type_name == "edge" and not self.edge_group_tag)
+            or (entity_type_name == "body" and not self.body_group_tag)
+        ):
+            raise Flow360ValueError(
+                f"Renaming failed: Could not find {entity_type_name} grouping info in the draft's simulation settings."
+                "Please group them first before renaming the entities."
+            )
+
+        matched_entities = self.internal_registry.find_by_naming_pattern(
+            pattern=current_name_pattern
+        )
+        if entity_type_name == "body":
+            matched_entities = [
+                entity for entity in matched_entities if isinstance(entity, GeometryBodyGroup)
+            ]
+        if entity_type_name == "face":
+            matched_entities = [
+                entity for entity in matched_entities if isinstance(entity, Surface)
+            ]
+        if entity_type_name == "edge":
+            matched_entities = [entity for entity in matched_entities if isinstance(entity, Edge)]
+
+        matched_entities = sorted(
+            matched_entities,
+            key=lambda x: x.name,
+        )
+        if len(matched_entities) == 0:
+            raise Flow360ValueError(
+                f"Renaming failed: No entity is found to match the input name pattern: {current_name_pattern}."
+            )
+
+        for idx, entity in enumerate(matched_entities):
+            new_name = (
+                f"{new_name_prefix}_{(idx+1):04d}" if len(matched_entities) > 1 else new_name_prefix
+            )
+            if self.internal_registry.find_by_naming_pattern(new_name):
+                raise Flow360ValueError(
+                    f"Renaming failed: An entity with the new name: {new_name} already exists."
+                )
+            with model_attribute_unlock(entity, "name"):
+                entity.name = new_name
+
+    def rename_edges(self, current_name_pattern: str, new_name_prefix: str):
+        """
+        Rename the edge in the current edge group
+
+        Parameters
+        ----------
+        current_name_pattern:
+            The current name of a single edge or the name pattern of the edges
+
+        new_name_prefix:
+            The new name of a single edge or the new name prefix of the edges
+        """
+        self._rename_entity(
+            entity_type_name="edge",
+            current_name_pattern=current_name_pattern,
+            new_name_prefix=new_name_prefix,
+        )
+
+    def rename_surfaces(self, current_name_pattern: str, new_name_prefix: str):
+        """
+        Rename the face in the current face group
+
+        Parameters
+        ----------
+        current_name_pattern:
+            The current name of a single face or the name pattern of the faces
+
+        new_name_prefix:
+            The new name of a single face or the new name prefix of the faces
+        """
+        self._rename_entity(
+            entity_type_name="face",
+            current_name_pattern=current_name_pattern,
+            new_name_prefix=new_name_prefix,
+        )
+
+    def rename_body_groups(self, current_name_pattern: str, new_name_prefix: str):
+        """
+        Rename the body in the current body group
+
+        Parameters
+        ----------
+        current_name_pattern:
+            The current name of a single body or the name pattern of the bodies
+
+        new_name_prefix:
+            The new name of a single body or the new name prefix of the bodies
+        """
+        self._rename_entity(
+            entity_type_name="body",
+            current_name_pattern=current_name_pattern,
+            new_name_prefix=new_name_prefix,
+        )
 
     def __getitem__(self, key: str):
         """
@@ -414,7 +556,7 @@ class Geometry(AssetBase):
 
         if hasattr(self, "internal_registry") is False or self.internal_registry is None:
             raise Flow360ValueError(
-                "The faces/edges in geometry are not grouped yet."
+                "The faces/edges/bodies in geometry are not grouped yet."
                 "Please use `group_faces_by_tag` or `group_edges_by_tag` function to group them first."
             )
             # Note: Or we assume group default by just FaceID and EdgeID? Not sure if this is actually useful.
