@@ -2,6 +2,7 @@
 
 from __future__ import annotations
 
+import copy
 import hashlib
 import json
 from itertools import chain
@@ -30,6 +31,32 @@ DISCRIMINATOR_NAMES = [
 ]
 
 
+def _preprocess_nested_list(value, required_by, params, exclude, registry_lookup):
+    new_list = []
+    for i, item in enumerate(value):
+        # Extend the 'required_by' path with the current index.
+        new_required_by = required_by + [f"{i}"]
+        if isinstance(item, list):
+            # Recursively process nested lists.
+            new_list.append(
+                _preprocess_nested_list(item, new_required_by, params, exclude, registry_lookup)
+            )
+        elif isinstance(item, Flow360BaseModel):
+            # Process Flow360BaseModel instances.
+            new_list.append(
+                item.preprocess(
+                    params=params,
+                    required_by=new_required_by,
+                    exclude=exclude,
+                    registry_lookup=registry_lookup,
+                )
+            )
+        else:
+            # Return item unchanged if it doesn't need processing.
+            new_list.append(item)
+    return new_list
+
+
 def snake_to_camel(string: str) -> str:
     """
     Convert a snake_case string to camelCase.
@@ -142,8 +169,9 @@ class Flow360BaseModel(pd.BaseModel):
     )
 
     def __setattr__(self, name, value):
-        if name in self.model_fields:
-            is_frozen = self.model_fields[name].frozen
+        # pylint: disable=unsupported-membership-test,  unsubscriptable-object
+        if name in self.__class__.model_fields:
+            is_frozen = self.__class__.model_fields[name].frozen
             if is_frozen is not None and is_frozen is True:
                 raise ValueError(f"Cannot modify immutable/frozen fields: {name}")
         super().__setattr__(name, value)
@@ -214,6 +242,7 @@ class Flow360BaseModel(pd.BaseModel):
     @classmethod
     def _get_field_context(cls, info, context_key):
         if info.field_name is not None:
+            # pylint:disable = unsubscriptable-object
             field_info = cls.model_fields[info.field_name]
             if isinstance(field_info.json_schema_extra, dict):
                 return field_info.json_schema_extra.get(context_key)
@@ -256,7 +285,10 @@ class Flow360BaseModel(pd.BaseModel):
         if need_to_rebuild is True:
             for mode, method in validators:
                 info = FieldValidatorDecoratorInfo(
-                    fields=tuple(fields_to_validate), mode=mode, check_fields=None
+                    fields=tuple(fields_to_validate),
+                    mode=mode,
+                    check_fields=None,
+                    json_schema_input_type=Any,
                 )
                 deco = Decorator.build(cls, cls_var_name=method, info=info, shim=None)
                 cls.__pydantic_decorators__.field_validators[method] = deco
@@ -449,7 +481,7 @@ class Flow360BaseModel(pd.BaseModel):
         -------
         >>> params._to_json(filename='folder/flow360.json') # doctest: +SKIP
         """
-        json_string = self.model_dump_json(**kwargs)
+        json_string = self.model_dump_json(exclude_none=True, **kwargs)
         model_dict = json.loads(json_string)
         if self.model_config["include_hash"] is True:
             model_dict["hash"] = self._calculate_hash(model_dict)
@@ -490,7 +522,7 @@ class Flow360BaseModel(pd.BaseModel):
         -------
         >>> params._to_yaml(filename='folder/flow360.yaml') # doctest: +SKIP
         """
-        json_string = self.model_dump_json(**kwargs)
+        json_string = self.model_dump_json(exclude_none=True, **kwargs)
         model_dict = json.loads(json_string)
         if self.model_config["include_hash"]:
             model_dict["hash"] = self._calculate_hash(model_dict)
@@ -539,7 +571,7 @@ class Flow360BaseModel(pd.BaseModel):
 
         for property_name, value in chain(self_dict.items(), additional_fields.items()):
             loc_name = property_name
-            field = self.model_fields.get(property_name)
+            field = self.__class__.model_fields.get(property_name)
             if field is not None and field.alias is not None:
                 loc_name = field.alias
             if need_conversion(value) and property_name not in exclude:
@@ -559,7 +591,7 @@ class Flow360BaseModel(pd.BaseModel):
                 value.units.registry = registry_lookup.registry
                 solver_values[property_name] = value.in_base(unit_system="flow360_v2")
             else:
-                solver_values[property_name] = value
+                solver_values[property_name] = copy.copy(value)
 
         return solver_values
 
@@ -616,7 +648,7 @@ class Flow360BaseModel(pd.BaseModel):
             if property_name in exclude:
                 continue
             loc_name = property_name
-            field = self.model_fields.get(property_name)
+            field = self.__class__.model_fields.get(property_name)
             if field is not None and field.alias is not None:
                 loc_name = field.alias
             if isinstance(value, Flow360BaseModel):
@@ -627,13 +659,9 @@ class Flow360BaseModel(pd.BaseModel):
                     registry_lookup=registry_lookup,
                 )
             elif isinstance(value, list):
-                for i, item in enumerate(value):
-                    if isinstance(item, Flow360BaseModel):
-                        solver_values[property_name][i] = item.preprocess(
-                            params=params,
-                            required_by=[*required_by, loc_name, f"{i}"],
-                            exclude=exclude,
-                            registry_lookup=registry_lookup,
-                        )
+                # Use the helper to handle nested lists.
+                solver_values[property_name] = _preprocess_nested_list(
+                    value, [loc_name], params, exclude, registry_lookup
+                )
 
         return self.__class__(**solver_values)
