@@ -3,7 +3,7 @@ Contains basically only boundary conditons for now. In future we can add new mod
 """
 
 from abc import ABCMeta
-from typing import Annotated, Literal, Optional, Union
+from typing import Annotated, Dict, Literal, Optional, Union
 
 import pydantic as pd
 
@@ -36,10 +36,13 @@ from flow360.component.simulation.unit_system import (
     MassFlowRateType,
     PressureType,
 )
-from flow360.component.simulation.utils import is_instance_of_type_in_union
-from flow360.component.simulation.validation_utils import (
+from flow360.component.simulation.validation.validation_context import (
+    get_validation_info,
+)
+from flow360.component.simulation.validation.validation_utils import (
     check_deleted_surface_in_entity_list,
     check_deleted_surface_pair,
+    check_symmetric_boundary_existence,
 )
 
 # pylint: disable=fixme
@@ -47,6 +50,16 @@ from flow360.component.simulation.validation_utils import (
 from flow360.component.types import Axis
 
 
+class EntityListAllowingGhost(EntityList):
+    """Entity list with customized validators for ghost entities"""
+
+    @pd.field_validator("stored_entities", mode="after")
+    @classmethod
+    def ghost_entity_validator(cls, value):
+        """Run all validators"""
+        return check_symmetric_boundary_existence(value)
+
+
 class BoundaryBase(Flow360BaseModel, metaclass=ABCMeta):
     """Boundary base"""
 
@@ -60,6 +73,8 @@ class BoundaryBase(Flow360BaseModel, metaclass=ABCMeta):
     @classmethod
     def ensure_surface_existence(cls, value):
         """Ensure all boundaries will be present after mesher"""
+        # pylint: disable=fixme
+        # TODO: This should have been moved to EntityListAllowingGhost?
         return check_deleted_surface_in_entity_list(value)
 
 
@@ -121,7 +136,6 @@ class TotalPressure(Flow360BaseModel):
 
     >>> fl.TotalPressure(
     ...     value = 1.04e6 * fl.u.Pa,
-    ...     velocity_direction = (1, 0, 0),
     ... )
 
     ====
@@ -130,11 +144,6 @@ class TotalPressure(Flow360BaseModel):
     type_name: Literal["TotalPressure"] = pd.Field("TotalPressure", frozen=True)
     # pylint: disable=no-member
     value: PressureType.Positive = pd.Field(description="The total pressure value.")
-    velocity_direction: Optional[Axis] = pd.Field(
-        None,
-        description="Direction of the incoming flow. Must be a unit vector pointing "
-        + "into the volume. If unspecified, the direction will be normal to the surface.",
-    )
 
 
 class Pressure(SingleAttributeModel):
@@ -263,6 +272,7 @@ class WallRotation(Flow360BaseModel):
     ...     ),
     ...     use_wall_function=True,
     ... )
+
     ====
     """
 
@@ -271,6 +281,7 @@ class WallRotation(Flow360BaseModel):
     axis: Axis = pd.Field(description="The axis of rotation.")
     angular_velocity: AngularVelocityType = pd.Field("The value of the angular velocity.")
     type_name: Literal["WallRotation"] = pd.Field("WallRotation", frozen=True)
+    private_attribute_circle_mode: Optional[dict] = pd.Field(None)
 
 
 ##########################################
@@ -337,7 +348,7 @@ class Wall(BoundaryBase):
     ====
     """
 
-    name: Optional[str] = pd.Field(None, description="Name of the `Wall` boundary condition.")
+    name: Optional[str] = pd.Field("Wall", description="Name of the `Wall` boundary condition.")
     type: Literal["Wall"] = pd.Field("Wall", frozen=True)
     use_wall_function: bool = pd.Field(
         False,
@@ -359,18 +370,48 @@ class Wall(BoundaryBase):
         0 * u.m,
         description="Equivalent sand grain roughness height. Available only to `Fluid` zone boundaries.",
     )
+    private_attribute_dict: Optional[Dict] = pd.Field(None)
 
     @pd.model_validator(mode="after")
     def check_wall_function_conflict(self):
         """Check no setting is conflicting with the usage of wall function"""
         if self.use_wall_function is False:
             return self
-        if is_instance_of_type_in_union(self.velocity, WallVelocityModelTypes):
+        if isinstance(self.velocity, SlaterPorousBleed):
             raise ValueError(
                 f"Using `{type(self.velocity).__name__}` with wall function is not supported currently."
             )
         return self
 
+    @pd.field_validator("heat_spec", mode="after")
+    @classmethod
+    def _ensure_adiabatic_wall_for_liquid(cls, value):
+        """Allow only adiabatic wall when liquid operating condition is used"""
+        validation_info = get_validation_info()
+        if validation_info is None or validation_info.using_liquid_as_material is False:
+            return value
+        if isinstance(value, HeatFlux) and value.value == 0 * u.W / u.m**2:
+            return value
+        raise ValueError("Only adiabatic wall is allowed when using liquid as simulation material.")
+
+    @pd.field_validator("velocity", mode="after")
+    @classmethod
+    def _disable_expression_for_liquid(cls, value):
+        validation_info = get_validation_info()
+        if validation_info is None or validation_info.using_liquid_as_material is False:
+            return value
+
+        if isinstance(value, tuple):
+            if (
+                isinstance(value[0], str)
+                and isinstance(value[1], str)
+                and isinstance(value[2], str)
+            ):
+                raise ValueError(
+                    "Expression cannot be used when using liquid as simulation material."
+                )
+        return value
+
 
 class Freestream(BoundaryBaseWithTurbulenceQuantities):
     """
@@ -400,7 +441,9 @@ class Freestream(BoundaryBaseWithTurbulenceQuantities):
     ====
     """
 
-    name: Optional[str] = pd.Field(None, description="Name of the `Freestream` boundary condition.")
+    name: Optional[str] = pd.Field(
+        "Freestream", description="Name of the `Freestream` boundary condition."
+    )
     type: Literal["Freestream"] = pd.Field("Freestream", frozen=True)
     velocity: Optional[VelocityVectorType] = pd.Field(
         None,
@@ -408,11 +451,31 @@ class Freestream(BoundaryBaseWithTurbulenceQuantities):
         + ":py:attr:`AerospaceCondition.alpha` and :py:attr:`AerospaceCondition.beta` angles. "
         + "Optionally, an expression for each of the velocity components can be specified.",
     )
-    entities: EntityList[Surface, GhostSurface, GhostSphere, GhostCircularPlane] = pd.Field(
-        alias="surfaces",
-        description="List of boundaries with the `Freestream` boundary condition imposed.",
+    entities: EntityListAllowingGhost[Surface, GhostSurface, GhostSphere, GhostCircularPlane] = (
+        pd.Field(
+            alias="surfaces",
+            description="List of boundaries with the `Freestream` boundary condition imposed.",
+        )
     )
 
+    @pd.field_validator("velocity", mode="after")
+    @classmethod
+    def _disable_expression_for_liquid(cls, value):
+        validation_info = get_validation_info()
+        if validation_info is None or validation_info.using_liquid_as_material is False:
+            return value
+
+        if isinstance(value, tuple):
+            if (
+                isinstance(value[0], str)
+                and isinstance(value[1], str)
+                and isinstance(value[2], str)
+            ):
+                raise ValueError(
+                    "Expression cannot be used when using liquid as simulation material."
+                )
+        return value
+
 
 class Outflow(BoundaryBase):
     """
@@ -444,7 +507,9 @@ class Outflow(BoundaryBase):
     ====
     """
 
-    name: Optional[str] = pd.Field(None, description="Name of the `Outflow` boundary condition.")
+    name: Optional[str] = pd.Field(
+        "Outflow", description="Name of the `Outflow` boundary condition."
+    )
     type: Literal["Outflow"] = pd.Field("Outflow", frozen=True)
     spec: Union[Pressure, MassFlowRate, Mach] = pd.Field(
         discriminator="type_name",
@@ -467,8 +532,8 @@ class Inflow(BoundaryBaseWithTurbulenceQuantities):
       ...     total_temperature=300 * fl.u.K,
       ...     spec=fl.TotalPressure(
       ...         value = 1.028e6 * fl.u.Pa,
-      ...         velocity_direction = (1, 0, 0),
       ...     ),
+      ...     velocity_direction = (1, 0, 0),
       ... )
 
     - Define inflow boundary condition with mass flow rate:
@@ -480,6 +545,7 @@ class Inflow(BoundaryBaseWithTurbulenceQuantities):
       ...         value = 123 * fl.u.lb / fl.u.s,
       ...         ramp_steps = 10,
       ...     ),
+      ...     velocity_direction = (1, 0, 0),
       ... )
 
     - Define inflow boundary condition with turbulence quantities:
@@ -495,7 +561,7 @@ class Inflow(BoundaryBaseWithTurbulenceQuantities):
     ====
     """
 
-    name: Optional[str] = pd.Field(None, description="Name of the `Inflow` boundary condition.")
+    name: Optional[str] = pd.Field("Inflow", description="Name of the `Inflow` boundary condition.")
     type: Literal["Inflow"] = pd.Field("Inflow", frozen=True)
     # pylint: disable=no-member
     total_temperature: AbsoluteTemperatureType = pd.Field(
@@ -505,6 +571,11 @@ class Inflow(BoundaryBaseWithTurbulenceQuantities):
         discriminator="type_name",
         description="Specify the total pressure or the mass flow rate at the `Inflow` boundary.",
     )
+    velocity_direction: Optional[Axis] = pd.Field(
+        None,
+        description="Direction of the incoming flow. Must be a unit vector pointing "
+        + "into the volume. If unspecified, the direction will be normal to the surface.",
+    )
 
 
 class SlipWall(BoundaryBase):
@@ -531,9 +602,11 @@ class SlipWall(BoundaryBase):
     ====
     """
 
-    name: Optional[str] = pd.Field(None, description="Name of the `SlipWall` boundary condition.")
+    name: Optional[str] = pd.Field(
+        "Slip wall", description="Name of the `SlipWall` boundary condition."
+    )
     type: Literal["SlipWall"] = pd.Field("SlipWall", frozen=True)
-    entities: EntityList[Surface, GhostSurface, GhostCircularPlane] = pd.Field(
+    entities: EntityListAllowingGhost[Surface, GhostSurface, GhostCircularPlane] = pd.Field(
         alias="surfaces",
         description="List of boundaries with the :code:`SlipWall` boundary condition imposed.",
     )
@@ -561,10 +634,10 @@ class SymmetryPlane(BoundaryBase):
     """
 
     name: Optional[str] = pd.Field(
-        None, description="Name of the `SymmetryPlane` boundary condition."
+        "Symmetry", description="Name of the `SymmetryPlane` boundary condition."
     )
     type: Literal["SymmetryPlane"] = pd.Field("SymmetryPlane", frozen=True)
-    entities: EntityList[Surface, GhostSurface, GhostCircularPlane] = pd.Field(
+    entities: EntityListAllowingGhost[Surface, GhostSurface, GhostCircularPlane] = pd.Field(
         alias="surfaces",
         description="List of boundaries with the `SymmetryPlane` boundary condition imposed.",
     )
@@ -598,7 +671,9 @@ class Periodic(Flow360BaseModel):
     ====
     """
 
-    name: Optional[str] = pd.Field(None, description="Name of the `Periodic` boundary condition.")
+    name: Optional[str] = pd.Field(
+        "Periodic", description="Name of the `Periodic` boundary condition."
+    )
     type: Literal["Periodic"] = pd.Field("Periodic", frozen=True)
     entity_pairs: UniqueItemList[SurfacePair] = pd.Field(
         alias="surface_pairs", description="List of matching pairs of :class:`~flow360.Surface`. "
