@@ -2,6 +2,7 @@
 
 # pylint: disable=too-many-lines
 import os
+import re
 from abc import ABCMeta
 from typing import Annotated, Dict, List, Literal, Optional, Union
 
@@ -10,7 +11,10 @@ import pydantic as pd
 import flow360.component.simulation.units as u
 from flow360.component.simulation.framework.base_model import Flow360BaseModel
 from flow360.component.simulation.framework.entity_base import EntityList
-from flow360.component.simulation.framework.expressions import StringExpression
+from flow360.component.simulation.framework.expressions import (
+    StringExpression,
+    validate_angle_expression_of_t_seconds,
+)
 from flow360.component.simulation.framework.multi_constructor_model_base import (
     MultiConstructorBaseModel,
 )
@@ -58,7 +62,11 @@ from flow360.component.simulation.unit_system import (
     PressureType,
     u,
 )
-from flow360.component.simulation.validation_utils import (
+from flow360.component.simulation.user_code.core.types import ValueOrExpression
+from flow360.component.simulation.validation.validation_context import (
+    get_validation_info,
+)
+from flow360.component.simulation.validation.validation_utils import (
     _validator_append_instance_name,
 )
 
@@ -85,6 +93,25 @@ class AngleExpression(SingleAttributeModel):
         description="The expression defining the rotation angle as a function of time."
     )
 
+    @pd.field_validator("value", mode="after")
+    @classmethod
+    def _validate_angle_expression(cls, value):
+        errors = validate_angle_expression_of_t_seconds(value)
+        if errors:
+            raise ValueError(" | ".join(errors))
+        return value
+
+    def preprocess(self, **kwargs):
+        # locate t_seconds and convert it to (t*flow360_time_to_seconds)
+        params = kwargs.get("params")
+        one_sec_to_flow360_time = params.convert_unit(
+            value=1 * u.s, target_system="flow360_v2"  # pylint:disable=no-member
+        )
+        flow360_time_to_seconds_expression = f"({1.0/one_sec_to_flow360_time.value} * t)"
+        self.value = re.sub(r"\bt_seconds\b", flow360_time_to_seconds_expression, self.value)
+
+        return super().preprocess(**kwargs)
+
 
 class AngularVelocity(SingleAttributeModel):
     """
@@ -101,7 +128,9 @@ class AngularVelocity(SingleAttributeModel):
     """
 
     type_name: Literal["AngularVelocity"] = pd.Field("AngularVelocity", frozen=True)
-    value: AngularVelocityType = pd.Field(description="The value of the angular velocity.")
+    value: ValueOrExpression[AngularVelocityType] = pd.Field(
+        description="The value of the angular velocity."
+    )
 
 
 class FromUserDefinedDynamics(Flow360BaseModel):
@@ -129,7 +158,7 @@ class FromUserDefinedDynamics(Flow360BaseModel):
 
 class ExpressionInitialConditionBase(Flow360BaseModel):
     """
-    :class:`ExpressionInitialCondition` class for specifying the intial conditions of
+    :class:`ExpressionInitialCondition` class for specifying the initial conditions of
     :py:attr:`Fluid.initial_condition`.
     """
 
@@ -173,6 +202,18 @@ class NavierStokesInitialCondition(ExpressionInitialConditionBase):
     w: StringExpression = pd.Field("w", description="Z-direction velocity")
     p: StringExpression = pd.Field("p", description="Pressure")
 
+    @pd.field_validator("rho", "u", "v", "w", "p", mode="after")
+    @classmethod
+    def _disable_expression_for_liquid(cls, value, info: pd.ValidationInfo):
+        validation_info = get_validation_info()
+        if validation_info is None or validation_info.using_liquid_as_material is False:
+            return value
+
+        # pylint:disable = unsubscriptable-object
+        if value != cls.model_fields[info.field_name].get_default():
+            raise ValueError("Expression cannot be used when using liquid as simulation material.")
+        return value
+
 
 class NavierStokesModifiedRestartSolution(NavierStokesInitialCondition):
     type_name: Literal["NavierStokesModifiedRestartSolution"] = pd.Field(
@@ -255,7 +296,7 @@ class Fluid(PDEModelBase):
         + ":class:`TransitionModelSolver` documentation.",
     )
 
-    material: FluidMaterialTypes = pd.Field(Air(), description="The material propetry of fluid.")
+    material: FluidMaterialTypes = pd.Field(Air(), description="The material property of fluid.")
 
     initial_condition: Union[NavierStokesModifiedRestartSolution, NavierStokesInitialCondition] = (
         pd.Field(
@@ -414,7 +455,7 @@ class ActuatorDisk(Flow360BaseModel):
         description="The force per area input for the `ActuatorDisk` model. "
         + "See :class:`ForcePerArea` documentation."
     )
-    name: Optional[str] = pd.Field(None, description="Name of the `ActuatorDisk` model.")
+    name: Optional[str] = pd.Field("Actuator disk", description="Name of the `ActuatorDisk` model.")
     type: Literal["ActuatorDisk"] = pd.Field("ActuatorDisk", frozen=True)
 
 
@@ -513,7 +554,7 @@ class BETSingleInputFileBaseModel(Flow360BaseModel, metaclass=ABCMeta):
 
         file_content = get_file_content(input_data["file_path"])
 
-        return {"file_path": input_data["file_path"], "content": file_content}
+        return {"file_path": os.path.basename(input_data["file_path"]), "content": file_content}
 
 
 class AuxiliaryPolarFile(BETSingleInputFileBaseModel):
@@ -556,7 +597,11 @@ class BETSingleMultiFileBaseModel(Flow360BaseModel, metaclass=ABCMeta):
             polar_file_obj_list.append(
                 [{"file_path": os.path.join(file_dir, file_name)} for file_name in file_name_list]
             )
-        return {"file_path": file_path, "content": file_content, "polar_files": polar_file_obj_list}
+        return {
+            "file_path": os.path.basename(file_path),
+            "content": file_content,
+            "polar_files": polar_file_obj_list,
+        }
 
 
 class XROTORFile(BETSingleInputFileBaseModel):
@@ -584,6 +629,7 @@ BETFileTypes = Annotated[
 class BETDiskCache(Flow360BaseModel):
     """[INTERNAL] Cache for BETDisk inputs"""
 
+    name: Optional[str] = None
     file: Optional[BETFileTypes] = None
     rotation_direction_rule: Optional[Literal["leftHand", "rightHand"]] = None
     omega: Optional[AngularVelocityType.NonNegative] = None
@@ -592,7 +638,6 @@ class BETDiskCache(Flow360BaseModel):
     entities: Optional[EntityList[Cylinder]] = None
     angle_unit: Optional[AngleType] = None
     length_unit: Optional[LengthType.NonNegative] = None
-    mesh_unit: Optional[LengthType.NonNegative] = None
     number_of_blades: Optional[pd.StrictInt] = None
     initial_blade_direction: Optional[Axis] = None
     blade_line_chord: Optional[LengthType.NonNegative] = None
@@ -604,8 +649,8 @@ class BETDisk(MultiConstructorBaseModel):
     To generate the sectional polars the BET translators can be used which are
     outlined :ref:`here <BET_Translators>`
     with best-practices for the sectional polars inputs available :ref:`here <secPolars_bestPractices>`.
-    A case study of the XV-15 rotor using the steady BET Disk method is available
-    in :ref:`Case Studies <XV15BETDisk_caseStudy>`.
+    A validation study of the XV-15 rotor using the steady BET Disk method is available
+    in :ref:`Validation Studies <XV15BETDiskValidationStudy>`.
     Because a transient BET Line simulation is simply a time-accurate version of a steady-state
     BET Disk simulation, most of the parameters below are applicable to both methods.
 
@@ -636,7 +681,7 @@ class BETDisk(MultiConstructorBaseModel):
     ====
     """
 
-    name: Optional[str] = pd.Field(None, description="Name of the `BETDisk` model.")
+    name: Optional[str] = pd.Field("BET disk", description="Name of the `BETDisk` model.")
     type: Literal["BETDisk"] = pd.Field("BETDisk", frozen=True)
     type_name: Literal["BETDisk"] = pd.Field("BETDisk", frozen=True)
     entities: EntityList[Cylinder] = pd.Field(alias="volumes")
@@ -670,34 +715,42 @@ class BETDisk(MultiConstructorBaseModel):
         "inf",
         description="Dimensional distance between blade tip and solid bodies to "
         + "define a :ref:`tip loss factor <TipGap>`.",
+        frozen=True,
     )
     mach_numbers: List[pd.NonNegativeFloat] = pd.Field(
         description="Mach numbers associated with airfoil polars provided "
-        + "in :class:`BETDiskSectionalPolar`."
+        + "in :class:`BETDiskSectionalPolar`.",
+        frozen=True,
     )
     reynolds_numbers: List[pd.PositiveFloat] = pd.Field(
         description="Reynolds numbers associated with the airfoil polars "
-        + "provided in :class:`BETDiskSectionalPolar`."
+        + "provided in :class:`BETDiskSectionalPolar`.",
+        frozen=True,
     )
     alphas: AngleType.Array = pd.Field(
         description="Alphas associated with airfoil polars provided in "
-        + ":class:`BETDiskSectionalPolar`."
+        + ":class:`BETDiskSectionalPolar`.",
+        frozen=True,
     )
     twists: List[BETDiskTwist] = pd.Field(
         description="A list of :class:`BETDiskTwist` objects specifying the twist in degrees as a "
-        + "function of radial location."
+        + "function of radial location.",
+        frozen=True,
     )
     chords: List[BETDiskChord] = pd.Field(
         description="A list of :class:`BETDiskChord` objects specifying the blade chord as a function "
-        + "of the radial location. "
+        + "of the radial location. ",
+        frozen=True,
     )
     sectional_polars: List[BETDiskSectionalPolar] = pd.Field(
         description="A list of :class:`BETDiskSectionalPolar` objects for every radial location specified in "
-        + ":py:attr:`sectional_radiuses`."
+        + ":py:attr:`sectional_radiuses`.",
+        frozen=True,
     )
     sectional_radiuses: LengthType.NonNegativeArray = pd.Field(
         description="A list of the radial locations in grid units at which :math:`C_l` "
-        + "and :math:`C_d` are specified in :class:`BETDiskSectionalPolar`."
+        + "and :math:`C_d` are specified in :class:`BETDiskSectionalPolar`.",
+        frozen=True,
     )
 
     private_attribute_input_cache: BETDiskCache = BETDiskCache()
@@ -741,6 +794,26 @@ class BETDisk(MultiConstructorBaseModel):
         """validate dimension of 3d coefficients in polars"""
         return _check_bet_disk_3d_coefficients_in_polars(self)
 
+    @pd.field_validator(
+        "name",
+        "rotation_direction_rule",
+        "omega",
+        "chord_ref",
+        "n_loading_nodes",
+        "number_of_blades",
+        "entities",
+        "initial_blade_direction",
+        mode="after",
+    )
+    @classmethod
+    def _update_input_cache(cls, value, info: pd.ValidationInfo):
+        setattr(
+            info.data["private_attribute_input_cache"],
+            info.field_name,
+            value if info.field_name != "entities" else value.stored_entities,
+        )
+        return value
+
     # pylint: disable=too-many-arguments, no-self-argument, not-callable
     @MultiConstructorBaseModel.model_constructor
     @pd.validate_call
@@ -757,6 +830,7 @@ class BETDisk(MultiConstructorBaseModel):
         angle_unit: AngleType,
         initial_blade_direction: Optional[Axis] = None,
         blade_line_chord: LengthType.NonNegative = 0 * u.m,
+        name: str = "BET disk",
     ):
         """Constructs a :class: `BETDisk` instance from a given C81 file and additional inputs.
 
@@ -822,6 +896,7 @@ class BETDisk(MultiConstructorBaseModel):
             angle_unit=angle_unit,
             length_unit=length_unit,
             number_of_blades=number_of_blades,
+            name=name,
         )
 
         return cls(**params)
@@ -841,6 +916,7 @@ class BETDisk(MultiConstructorBaseModel):
         angle_unit: AngleType,
         initial_blade_direction: Optional[Axis] = None,
         blade_line_chord: LengthType.NonNegative = 0 * u.m,
+        name: str = "BET disk",
     ):
         """Constructs a :class: `BETDisk` instance from a given DFDC file and additional inputs.
 
@@ -878,7 +954,7 @@ class BETDisk(MultiConstructorBaseModel):
         --------
         Create a BET disk with a DFDC file.
 
-        >>> param = fl.BETDisk.from_xrotor(
+        >>> param = fl.BETDisk.from_dfdc(
         ...     file=fl.DFDCFile(file_path="dfdc_xv15.case")),
         ...     rotation_direction_rule="leftHand",
         ...     omega=0.0046 * fl.u.deg / fl.u.s,
@@ -886,7 +962,6 @@ class BETDisk(MultiConstructorBaseModel):
         ...     n_loading_nodes=20,
         ...     entities=bet_cylinder,
         ...     length_unit=fl.u.m,
-        ...     mesh_unit=fl.u.m,
         ...     angle_unit=fl.u.deg,
         ... )
         """
@@ -902,6 +977,7 @@ class BETDisk(MultiConstructorBaseModel):
             entities=entities,
             angle_unit=angle_unit,
             length_unit=length_unit,
+            name=name,
         )
 
         return cls(**params)
@@ -922,6 +998,7 @@ class BETDisk(MultiConstructorBaseModel):
         number_of_blades: pd.StrictInt,
         initial_blade_direction: Optional[Axis],
         blade_line_chord: LengthType.NonNegative = 0 * u.m,
+        name: str = "BET disk",
     ):
         """Constructs a :class: `BETDisk` instance from a given XROTOR file and additional inputs.
 
@@ -989,6 +1066,7 @@ class BETDisk(MultiConstructorBaseModel):
             angle_unit=angle_unit,
             length_unit=length_unit,
             number_of_blades=number_of_blades,
+            name=name,
         )
 
         return cls(**params)
@@ -1008,6 +1086,7 @@ class BETDisk(MultiConstructorBaseModel):
         angle_unit: AngleType,
         initial_blade_direction: Optional[Axis] = None,
         blade_line_chord: LengthType.NonNegative = 0 * u.m,
+        name: str = "BET disk",
     ):
         """Constructs a :class: `BETDisk` instance from a given XROTOR file and additional inputs.
 
@@ -1068,6 +1147,7 @@ class BETDisk(MultiConstructorBaseModel):
             entities=entities,
             angle_unit=angle_unit,
             length_unit=length_unit,
+            name=name,
         )
 
         return cls(**params)
@@ -1109,7 +1189,7 @@ class Rotation(Flow360BaseModel):
     ====
     """
 
-    name: Optional[str] = pd.Field(None, description="Name of the `Rotation` model.")
+    name: Optional[str] = pd.Field("Rotation", description="Name of the `Rotation` model.")
     type: Literal["Rotation"] = pd.Field("Rotation", frozen=True)
     entities: EntityList[GenericVolume, Cylinder] = pd.Field(
         alias="volumes",
@@ -1193,7 +1273,7 @@ class PorousMedium(Flow360BaseModel):
     ====
     """
 
-    name: Optional[str] = pd.Field(None, description="Name of the `PorousMedium` model.")
+    name: Optional[str] = pd.Field("Porous medium", description="Name of the `PorousMedium` model.")
     type: Literal["PorousMedium"] = pd.Field("PorousMedium", frozen=True)
     entities: EntityList[GenericVolume, Box] = pd.Field(
         alias="volumes",
@@ -1228,6 +1308,22 @@ class PorousMedium(Flow360BaseModel):
                 )
         return value
 
+    @pd.field_validator("volumetric_heat_source", mode="after")
+    @classmethod
+    def _validate_volumetric_heat_source_for_liquid(
+        cls, value: Optional[Union[StringExpression, HeatSourceType]]
+    ):
+        """Disable the volumetric_heat_source when liquid operating condition is used"""
+        validation_info = get_validation_info()
+        if validation_info is None or validation_info.using_liquid_as_material is False:
+            return value
+        if value is not None:
+            raise ValueError(
+                "`volumetric_heat_source` cannot be setup under `PorousMedium` when using "
+                "liquid as simulation material."
+            )
+        return value
+
 
 VolumeModelTypes = Union[
     Fluid,
