@@ -4,7 +4,6 @@ Primitive type definitions for simulation entities.
 
 import re
 from abc import ABCMeta
-from enum import Enum
 from typing import Annotated, List, Literal, Optional, Tuple, Union, final
 
 import numpy as np
@@ -21,8 +20,10 @@ from flow360.component.simulation.framework.multi_constructor_model_base import
 )
 from flow360.component.simulation.framework.unique_list import UniqueStringList
 from flow360.component.simulation.unit_system import AngleType, AreaType, LengthType
-from flow360.component.simulation.utils import model_attribute_unlock
+from flow360.component.simulation.user_code.core.types import ValueOrExpression
+from flow360.component.simulation.utils import BoundingBoxType, model_attribute_unlock
 from flow360.component.types import Axis
+from flow360.exceptions import Flow360BoundaryMissingError
 
 
 def _get_boundary_full_name(surface_name: str, volume_mesh_meta: dict[str, dict]) -> str:
@@ -42,11 +43,14 @@ def _get_boundary_full_name(surface_name: str, volume_mesh_meta: dict[str, dict]
                 return existing_boundary_name
     if surface_name == "symmetric":
         # Provides more info when the symmetric boundary is not auto generated.
-        raise ValueError(
+        raise Flow360BoundaryMissingError(
             f"Parent zone not found for boundary: {surface_name}. "
-            + "It is likely that it was never auto generated because the condition is not met."
+            "It is likely that it was never auto generated because the condition is not met."
         )
-    raise ValueError(f"Parent zone not found for surface {surface_name}.")
+    raise Flow360BoundaryMissingError(
+        f"Parent zone not found for surface {surface_name}. "
+        "It may have been deleted due to overlapping with generated symmetry plane."
+    )
 
 
 def _check_axis_is_orthogonal(axis_pair: Tuple[Axis, Axis]) -> Tuple[Axis, Axis]:
@@ -87,16 +91,80 @@ class ReferenceGeometry(Flow360BaseModel):
     moment_length: Optional[Union[LengthType.Positive, LengthType.PositiveVector]] = pd.Field(
         None, description="The x, y, z component-wise moment reference lengths."
     )
-    area: Optional[AreaType.Positive] = pd.Field(
+    area: Optional[ValueOrExpression[AreaType.Positive]] = pd.Field(
         None, description="The reference area of the geometry."
     )
+    private_attribute_area_settings: Optional[dict] = pd.Field(None)
 
 
 class Transformation(Flow360BaseModel):
-    """Used in preprocess()/translator to meshing param for volume meshing interface"""
+    """Transformation that will be applied to a body group."""
 
-    axis_of_rotation: Optional[Axis] = pd.Field()
-    angle_of_rotation: Optional[AngleType] = pd.Field()
+    type_name: Literal["BodyGroupTransformation"] = pd.Field("BodyGroupTransformation", frozen=True)
+
+    origin: LengthType.Point = pd.Field(  # pylint:disable=no-member
+        (0, 0, 0) * u.m,  # pylint:disable=no-member
+        description="The origin for geometry transformation in the order of scale,"
+        " rotation and translation.",
+    )
+
+    axis_of_rotation: Axis = pd.Field((1, 0, 0))
+    angle_of_rotation: AngleType = pd.Field(0 * u.deg)  # pylint:disable=no-member
+
+    scale: Tuple[PositiveFloat, PositiveFloat, PositiveFloat] = pd.Field((1, 1, 1))
+
+    translation: LengthType.Point = pd.Field((0, 0, 0) * u.m)  # pylint:disable=no-member
+
+    private_attribute_matrix: Optional[list[float]] = pd.Field(None)
+
+    def get_transformation_matrix(self) -> np.ndarray:
+        """
+        Find 3(row)x4(column) transformation matrix and store as row major.
+        Applies to vector of [x, y, z, 1] in project length unit.
+        """
+        # pylint:disable=no-member
+        error_msg = "[Internal] `{}` is dimensioned. Use get_transformation_matrix() after non-dimensionalization!"
+        assert str(self.origin.units) == "flow360_length_unit", error_msg.format("origin")
+        assert str(self.translation.units) == "flow360_length_unit", error_msg.format("translation")
+        origin_array = np.asarray(self.origin.value)
+        translation_array = np.asarray(self.translation.value)
+
+        axis = np.asarray(self.axis_of_rotation, dtype=np.float64)
+        angle = self.angle_of_rotation.to("rad").v.item()
+
+        axis = axis / np.linalg.norm(axis)
+
+        rotation_scale_matrix = rotation_matrix_from_axis_and_angle(axis, angle) * np.array(
+            self.scale
+        )
+        final_translation = -rotation_scale_matrix @ origin_array + origin_array + translation_array
+
+        return np.hstack([rotation_scale_matrix, final_translation[:, np.newaxis]])
+
+
+class GeometryBodyGroup(EntityBase):
+    """
+    :class:`GeometryBodyGroup` represents a collection of bodies that are grouped for transformation.
+    """
+
+    private_attribute_registry_bucket_name: Literal["GeometryBodyGroupEntityType"] = (
+        "GeometryBodyGroupEntityType"
+    )
+    private_attribute_tag_key: str = pd.Field(
+        description="The tag/attribute string used to group bodies.",
+    )
+    private_attribute_entity_type_name: Literal["GeometryBodyGroup"] = pd.Field(
+        "GeometryBodyGroup", frozen=True
+    )
+    private_attribute_sub_components: List[str] = pd.Field(
+        description="A list of body IDs which constitutes the current body group"
+    )
+    private_attribute_color: Optional[str] = pd.Field(
+        None, description="Color used for visualization"
+    )
+    transformation: Transformation = pd.Field(
+        Transformation(), description="The transformation performed on the body group"
+    )
 
 
 class _VolumeEntityBase(EntityBase, metaclass=ABCMeta):
@@ -293,7 +361,6 @@ class Box(MultiConstructorBaseModel, _VolumeEntityBase):
         center: LengthType.Point,
         size: LengthType.PositiveVector,
         axes: OrthogonalAxes,
-        **kwargs,
     ):
         """
         Construct box from principal axes
@@ -324,7 +391,6 @@ class Box(MultiConstructorBaseModel, _VolumeEntityBase):
             size=size,
             axis_of_rotation=tuple(axis),
             angle_of_rotation=angle * u.rad,
-            **kwargs,
         )
 
     @pd.model_validator(mode="after")
@@ -398,37 +464,17 @@ class Cylinder(_VolumeEntityBase):
         return self
 
 
-class _SurfaceIssueEnums(str, Enum):
+class SurfacePrivateAttributes(Flow360BaseModel):
     """
-    Enums for indicating that there is something wrong/special about the surface.
-
-    +-------------------+--------------------+--------------------+
-    | If my sub faces...| Issue should be    | Conflict with when |
-    |                   | predicted as       | using              |
-    +-------------------+--------------------+--------------------+
-    | All overlaps with | overlap_half_model_| Auto and Quasi     |
-    | the HalfModel Symm| symmetric          |                    |
-    +-------------------+--------------------+--------------------+
-    | All overlaps with | overlap_quasi_3d_  | Quasi              |
-    | the Non Half      | symmetric          |                    |
-    | Model (the other  |                    |                    |
-    | Q3D) Symm. Or not |                    |                    |
-    | half model at all.|                    |                    |
-    +-------------------+--------------------+--------------------+
-    | Some on HalfModel | overlap_quasi_3d_  | Quasi              |
-    | Symm, Some on Non | symmetric          |                    |
-    | HalfModel Symm    |                    |                    |
-    +-------------------+--------------------+--------------------+
-    | Have some faces   | None               | None               |
-    | elsewhere         |                    |                    |
-    +-------------------+--------------------+--------------------+
-
-
+     Private attributes for Surface.
+    TODO: With the amount of private_attribute prefixes we have
+    TODO: Maybe it makes more sense to lump them together to save storage space?
     """
 
-    # pylint: disable=invalid-name
-    overlap_half_model_symmetric = "OverlapHalfModelSymmetric"
-    overlap_quasi_3d_symmetric = "OverlapQuasi3DSymmetric"
+    type_name: Literal["SurfacePrivateAttributes"] = pd.Field(
+        "SurfacePrivateAttributes", frozen=True
+    )
+    bounding_box: BoundingBoxType = pd.Field(description="Bounding box of the surface.")
 
 
 @final
@@ -451,53 +497,59 @@ class Surface(_SurfaceEntityBase):
     private_attribute_sub_components: Optional[List[str]] = pd.Field(
         [], description="The face ids in geometry that composed into this `Surface`."
     )
-    private_attribute_potential_issues: List[_SurfaceIssueEnums] = pd.Field(
-        [],
-        description="Issues (not necessarily problems) found on this `Surface` after inspection by "
-        "surface mesh / geometry pipeline. Used for determining the usability of the `Surface` instance"
-        " under certain features and/or its existence.",
+    private_attribute_color: Optional[str] = pd.Field(
+        None, description="Color used for visualization"
     )
+    private_attributes: Optional[SurfacePrivateAttributes] = pd.Field(None)
 
     # Note: private_attribute_id should not be `Optional` anymore.
     # B.C. Updater and geometry pipeline will populate it.
 
-    # pylint: disable=fixme
-    # TODO: With the amount of private_attribute prefixes we have
-    # TODO: here maybe it makes more sense to lump them together to save space?
+    def _overlaps(self, ghost_surface_center_y: Optional[float], length_tolerance: float) -> bool:
+        if self.private_attributes is None:
+            # Legacy cloud asset.
+            return False
+        # pylint: disable=no-member
+        my_bounding_box = self.private_attributes.bounding_box
+        if abs(my_bounding_box.ymax - ghost_surface_center_y) > length_tolerance:
+            return False
+        if abs(my_bounding_box.ymin - ghost_surface_center_y) > length_tolerance:
+            return False
+        return True
 
-    private_attribute_color: Optional[str] = pd.Field(
-        None, description="Front end storage for the color selected for this `Surface` entity."
-    )
-
-    # pylint: disable=fixme
-    # TODO: Should inherit from `ReferenceGeometry` but we do not support this from solver side.
-
-    def _will_be_deleted_by_mesher(self, farfield_method: Literal["auto", "quasi-3d"]) -> bool:
+    def _will_be_deleted_by_mesher(
+        # pylint: disable=too-many-arguments
+        self,
+        farfield_method: Optional[Literal["auto", "quasi-3d"]],
+        global_bounding_box: Optional[BoundingBoxType],
+        planar_face_tolerance: Optional[float],
+        half_model_symmetry_plane_center_y: Optional[float],
+        quasi_3d_symmetry_planes_center_y: Optional[tuple[float]],
+    ) -> bool:
         """
         Check against the automated farfield method and
         determine if the current `Surface` will be deleted by the mesher.
         """
-        if not self.private_attribute_potential_issues:
-            # If no special status reported or there is no auto farfield involved at all.
+        if global_bounding_box is None or planar_face_tolerance is None or farfield_method is None:
+            # VolumeMesh or Geometry/SurfaceMesh with legacy schema.
             return False
 
+        length_tolerance = global_bounding_box.largest_dimension * planar_face_tolerance
+
         if farfield_method == "auto":
-            # Single symmetry
-            # pylint: disable=unsupported-membership-test
-            return (
-                _SurfaceIssueEnums.overlap_half_model_symmetric
-                in self.private_attribute_potential_issues
-            )
+            if half_model_symmetry_plane_center_y is None:
+                # Legacy schema.
+                return False
+            return self._overlaps(half_model_symmetry_plane_center_y, length_tolerance)
 
         if farfield_method == "quasi-3d":
-            # Two symmetry
-            # pylint: disable=unsupported-membership-test
-            return (
-                _SurfaceIssueEnums.overlap_quasi_3d_symmetric
-                in self.private_attribute_potential_issues
-                or _SurfaceIssueEnums.overlap_half_model_symmetric
-                in self.private_attribute_potential_issues
-            )
+            if quasi_3d_symmetry_planes_center_y is None:
+                # Legacy schema.
+                return False
+            for plane_center_y in quasi_3d_symmetry_planes_center_y:
+                if self._overlaps(plane_center_y, length_tolerance):
+                    return True
+            return False
 
         raise ValueError(f"Unknown auto farfield generation method: {farfield_method}.")
 
@@ -526,6 +578,10 @@ class GhostSphere(_SurfaceEntityBase):
     center: Optional[List] = pd.Field(None, alias="center")
     max_radius: Optional[PositiveFloat] = pd.Field(None, alias="maxRadius")
 
+    def exists(self, _) -> bool:
+        """Ghost farfield always exists."""
+        return True
+
 
 # pylint: disable=missing-class-docstring
 @final
@@ -538,6 +594,42 @@ class GhostCircularPlane(_SurfaceEntityBase):
     max_radius: Optional[PositiveFloat] = pd.Field(None, alias="maxRadius")
     normal_axis: Optional[List] = pd.Field(None, alias="normalAxis")
 
+    def _get_existence_dependency(self, validation_info):
+        y_max = validation_info.global_bounding_box[1][1]
+        y_min = validation_info.global_bounding_box[0][1]
+
+        largest_dimension = -np.inf
+        for dim in range(3):
+            dimension = (
+                validation_info.global_bounding_box[1][dim]
+                - validation_info.global_bounding_box[0][dim]
+            )
+            largest_dimension = max(largest_dimension, dimension)
+
+        tolerance = largest_dimension * validation_info.planar_face_tolerance
+        return y_min, y_max, tolerance, largest_dimension
+
+    def exists(self, validation_info) -> bool:
+        """Mesher logic for symmetric plane existence."""
+
+        if self.name != "symmetric":
+            # Quasi-3D mode, no need to check existence.
+            return True
+
+        if validation_info is None:
+            raise ValueError("Validation info is required for GhostCircularPlane existence check.")
+
+        if validation_info.global_bounding_box is None:
+            # This likely means the user try to use mesher on old cloud resources.
+            # We cannot validate if symmetric exists so will let it pass. Pipeline will error out anyway.
+            return True
+        y_min, y_max, tolerance, _ = self._get_existence_dependency(validation_info)
+
+        positive_half = abs(y_min) < tolerance < y_max
+        negative_half = abs(y_max) < tolerance and y_min < -tolerance
+
+        return positive_half or negative_half
+
 
 class SurfacePair(Flow360BaseModel):
     """
