@@ -7,8 +7,12 @@ from __future__ import annotations
 from typing import Annotated, List, Optional, Union
 
 import pydantic as pd
+import unyt as u
 
-from flow360.component.simulation.conversion import unit_converter
+from flow360.component.simulation.conversion import (
+    LIQUID_IMAGINARY_FREESTREAM_MACH,
+    unit_converter,
+)
 from flow360.component.simulation.framework.base_model import Flow360BaseModel
 from flow360.component.simulation.framework.entity_registry import EntityRegistry
 from flow360.component.simulation.framework.param_utils import (
@@ -19,6 +23,7 @@ from flow360.component.simulation.framework.param_utils import (
     register_entity_list,
 )
 from flow360.component.simulation.framework.updater import updater
+from flow360.component.simulation.framework.updater_utils import Flow360Version
 from flow360.component.simulation.meshing_param.params import MeshingParams
 from flow360.component.simulation.meshing_param.volume_params import (
     AutomatedFarfield,
@@ -29,6 +34,7 @@ from flow360.component.simulation.models.volume_models import (
     ActuatorDisk,
     BETDisk,
     Fluid,
+    Solid,
     VolumeModelTypes,
 )
 from flow360.component.simulation.operating_condition.operating_condition import (
@@ -51,14 +57,23 @@ from flow360.component.simulation.primitives import (
 )
 from flow360.component.simulation.time_stepping.time_stepping import Steady, Unsteady
 from flow360.component.simulation.unit_system import (
+    AbsoluteTemperatureType,
+    DensityType,
     DimensionedTypes,
     LengthType,
+    MassType,
+    TimeType,
     UnitSystem,
     UnitSystemType,
+    VelocityType,
     is_flow360_unit,
     unit_system_manager,
     unyt_quantity,
 )
+from flow360.component.simulation.user_code.core.types import (
+    batch_get_user_variable_units,
+    get_post_processing_variables,
+)
 from flow360.component.simulation.user_defined_dynamics.user_defined_dynamics import (
     UserDefinedDynamic,
 )
@@ -75,11 +90,15 @@ from flow360.component.simulation.validation.validation_simulation_params import
     _check_consistency_hybrid_model_volume_output,
     _check_consistency_wall_function_and_surface_output,
     _check_duplicate_entities_in_models,
+    _check_duplicate_isosurface_names,
+    _check_duplicate_surface_usage,
+    _check_hybrid_model_to_use_zonal_enforcement,
     _check_low_mach_preconditioner_output,
     _check_numerical_dissipation_factor_output,
     _check_parent_volume_is_rotating,
     _check_time_average_output,
     _check_unsteadiness_to_use_hybrid_model,
+    _check_valid_models_for_liquid,
 )
 from flow360.component.utils import remove_properties_by_name
 from flow360.error_messages import (
@@ -87,6 +106,7 @@ from flow360.error_messages import (
     use_unit_system_for_simulation_msg,
 )
 from flow360.exceptions import Flow360ConfigurationError, Flow360RuntimeError
+from flow360.log import log
 from flow360.version import __version__
 
 from .validation.validation_context import (
@@ -96,6 +116,7 @@ from .validation.validation_context import (
     CaseField,
     ConditionalField,
     context_validator,
+    get_validation_info,
 )
 
 ModelTypes = Annotated[Union[VolumeModelTypes, SurfaceModelTypes], pd.Field(discriminator="type")]
@@ -131,20 +152,34 @@ class _ParamModelBase(Flow360BaseModel):
 
         return kwargs
 
+    @classmethod
+    def _get_version_from_dict(cls, model_dict: dict) -> str:
+        version = model_dict.get("version", None)
+        if version is None:
+            raise Flow360RuntimeError("Failed to find SimulationParams version from the input.")
+        return version
+
     @classmethod
     def _update_param_dict(cls, model_dict, version_to=__version__):
         """
         1. Find the version from the input dict.
         2. Update the input dict to `version_to` which by default is the current version.
+        3. If the simulation.json has higher version, then return the dict as is without modification.
+
+        Returns
+        -------
+        dict
+            The updated parameters dictionary.
+        bool
+            Whether the `model_dict` has higher version than `version_to` (AKA forward compatibility mode).
         """
-        version = model_dict.get("version", None)
-        if version is None:
-            raise Flow360RuntimeError("Failed to find SimulationParams version from the input.")
-        if version != version_to:
+        input_version = cls._get_version_from_dict(model_dict=model_dict)
+        forward_compatibility_mode = Flow360Version(input_version) > Flow360Version(version_to)
+        if not forward_compatibility_mode:
             model_dict = updater(
-                version_from=version, version_to=version_to, params_as_dict=model_dict
+                version_from=input_version, version_to=version_to, params_as_dict=model_dict
             )
-        return model_dict
+        return model_dict, forward_compatibility_mode
 
     @classmethod
     def _sanitize_params_dict(cls, model_dict):
@@ -171,7 +206,7 @@ class _ParamModelBase(Flow360BaseModel):
 
         model_dict = _ParamModelBase._sanitize_params_dict(model_dict)
         # When treating files/file like contents the updater will always be run.
-        model_dict = _ParamModelBase._update_param_dict(model_dict)
+        model_dict, _ = _ParamModelBase._update_param_dict(model_dict)
 
         unit_system = model_dict.get("unit_system")
 
@@ -184,6 +219,7 @@ class _ParamModelBase(Flow360BaseModel):
         """
         # When treating dicts the updater is skipped.
         kwargs = _ParamModelBase._init_check_unit_system(**kwargs)
+
         super().__init__(unit_system=unit_system_manager.current, **kwargs)
 
     # pylint: disable=super-init-not-called
@@ -263,6 +299,7 @@ class SimulationParams(_ParamModelBase):
 
     ##:: [INTERNAL USE ONLY] Private attributes that should not be modified manually.
     private_attribute_asset_cache: AssetCache = pd.Field(AssetCache(), frozen=True)
+    private_attribute_dict: Optional[dict] = pd.Field(None)
 
     # pylint: disable=arguments-differ
     def _preprocess(self, mesh_unit=None, exclude: list = None) -> SimulationParams:
@@ -351,7 +388,7 @@ class SimulationParams(_ParamModelBase):
     @pd.field_validator("models", mode="after")
     @classmethod
     def apply_default_fluid_settings(cls, v):
-        """apply default Fluid() settings if not found in models"""
+        """Apply default Fluid() settings if not found in mode`ls"""
         if v is None:
             v = []
         assert isinstance(v, list)
@@ -365,6 +402,37 @@ class SimulationParams(_ParamModelBase):
         """Ensure that all the parent volumes listed in the `Rotation` model are not static"""
         return _check_parent_volume_is_rotating(models)
 
+    @pd.field_validator("models", mode="after")
+    @classmethod
+    def check_valid_models_for_liquid(cls, models):
+        """Ensure that all the boundary conditions used are valid."""
+        return _check_valid_models_for_liquid(models)
+
+    @pd.field_validator("user_defined_dynamics", "user_defined_fields", mode="after")
+    @classmethod
+    def _disable_expression_for_liquid(cls, value, info: pd.ValidationInfo):
+        """Ensure that string expressions are disabled for liquid simulation."""
+        validation_info = get_validation_info()
+        if validation_info is None or validation_info.using_liquid_as_material is False:
+            return value
+        if value:
+            raise ValueError(
+                f"{info.field_name} cannot be used when using liquid as simulation material."
+            )
+        return value
+
+    @pd.field_validator("outputs", mode="after")
+    @classmethod
+    def check_duplicate_isosurface_names(cls, outputs):
+        """Check if we have isosurfaces with a duplicate name"""
+        return _check_duplicate_isosurface_names(outputs)
+
+    @pd.field_validator("outputs", mode="after")
+    @classmethod
+    def check_duplicate_surface_usage(cls, outputs):
+        """Disallow the same boundary/surface being used in multiple outputs"""
+        return _check_duplicate_surface_usage(outputs)
+
     @pd.field_validator("user_defined_fields", mode="after")
     @classmethod
     def check_duplicate_user_defined_fields(cls, v):
@@ -402,6 +470,11 @@ class SimulationParams(_ParamModelBase):
         """Only allow hybrid RANS-LES output field for unsteady simulations"""
         return _check_unsteadiness_to_use_hybrid_model(self)
 
+    @pd.model_validator(mode="after")
+    def check_hybrid_model_to_use_zonal_enforcement(self):
+        """Only allow LES/RANS zonal enforcement in hybrid RANS-LES mode"""
+        return _check_hybrid_model_to_use_zonal_enforcement(self)
+
     @pd.model_validator(mode="after")
     def check_unsteadiness_to_use_aero_acoustics(self):
         """Only allow Aero acoustics when using unsteady simulation"""
@@ -457,7 +530,7 @@ class SimulationParams(_ParamModelBase):
 
     def _update_entity_private_attrs(self, registry: EntityRegistry) -> EntityRegistry:
         """
-        Once the SimulationParams is set, extract and upate information
+        Once the SimulationParams is set, extract and update information
         into all used entities by parsing the params.
         """
 
@@ -483,6 +556,72 @@ class SimulationParams(_ParamModelBase):
 
         return registry
 
+    @property
+    def base_length(self) -> LengthType:
+        """Get base length unit for non-dimensionalization"""
+        # pylint:disable=no-member
+        return self.private_attribute_asset_cache.project_length_unit.to("m")
+
+    @property
+    def base_temperature(self) -> AbsoluteTemperatureType:
+        """Get base temperature unit for non-dimensionalization"""
+        # pylint:disable=no-member
+        if self.operating_condition.type_name == "LiquidOperatingCondition":
+            # Temperature in this condition has no effect because the thermal features will be disabled.
+            # Also the viscosity will be constant.
+            # pylint:disable = no-member
+            return 273 * u.K
+        return self.operating_condition.thermal_state.temperature.to("K")
+
+    @property
+    def base_velocity(self) -> VelocityType:
+        """Get base velocity unit for non-dimensionalization"""
+        # pylint:disable=no-member
+        if self.operating_condition.type_name == "LiquidOperatingCondition":
+            # Provides an imaginary "speed of sound"
+            # Resulting in a hardcoded freestream mach of `LIQUID_IMAGINARY_FREESTREAM_MACH`
+            # To ensure incompressible range.
+            # pylint: disable=protected-access
+            if self.operating_condition._evaluated_velocity_magnitude.value != 0:
+                return (
+                    self.operating_condition._evaluated_velocity_magnitude
+                    / LIQUID_IMAGINARY_FREESTREAM_MACH
+                ).to("m/s")
+            return (
+                self.operating_condition.reference_velocity_magnitude
+                / LIQUID_IMAGINARY_FREESTREAM_MACH
+            ).to("m/s")
+        return self.operating_condition.thermal_state.speed_of_sound.to("m/s")
+
+    @property
+    def base_density(self) -> DensityType:
+        """Get base density unit for non-dimensionalization"""
+        # pylint:disable=no-member
+        if self.operating_condition.type_name == "LiquidOperatingCondition":
+            return self.operating_condition.material.density.to("kg/m**3")
+        return self.operating_condition.thermal_state.density.to("kg/m**3")
+
+    @property
+    def base_mass(self) -> MassType:
+        """Get base mass unit for non-dimensionalization"""
+        return self.base_density * self.base_length**3
+
+    @property
+    def base_time(self) -> TimeType:
+        """Get base time unit for non-dimensionalization"""
+        return self.base_length / self.base_velocity
+
+    @property
+    def flow360_unit_system(self) -> u.UnitSystem:
+        """Get the unit system for non-dimensionalization"""
+        return u.UnitSystem(
+            name="flow360_nondim",
+            length_unit=self.base_length,
+            mass_unit=self.base_mass,
+            time_unit=self.base_time,
+            temperature_unit=self.base_temperature,
+        )
+
     @property
     def used_entity_registry(self) -> EntityRegistry:
         """
@@ -503,6 +642,7 @@ class SimulationParams(_ParamModelBase):
         """
         # pylint:disable=no-member
         used_entity_registry = self.used_entity_registry
+        # Below includes the Ghost entities.
         _update_entity_full_name(self, _SurfaceEntityBase, volume_mesh_meta_data)
         _update_entity_full_name(self, _VolumeEntityBase, volume_mesh_meta_data)
         _update_zone_boundaries_with_metadata(used_entity_registry, volume_mesh_meta_data)
@@ -514,6 +654,15 @@ class SimulationParams(_ParamModelBase):
         """
         return isinstance(self.time_stepping, Steady)
 
+    def has_solid(self):
+        """
+        returns True when SimulationParams has Solid model
+        """
+        if self.models is None:
+            return False
+        # pylint: disable=not-an-iterable
+        return any(isinstance(item, Solid) for item in self.models)
+
     def has_actuator_disks(self):
         """
         returns True when SimulationParams has ActuatorDisk disk
@@ -576,3 +725,53 @@ class SimulationParams(_ParamModelBase):
         returns True when SimulationParams has user defined dynamics
         """
         return self.user_defined_dynamics is not None and len(self.user_defined_dynamics) > 0
+
+    def display_output_units(self) -> None:
+        """
+        Display all the output units for UserVariables used in `outputs`.
+        """
+        if not self.outputs:
+            return
+
+        post_processing_variables = get_post_processing_variables(self)
+
+        # Sort for consistent behavior
+        post_processing_variables = sorted(post_processing_variables)
+        name_units_pair = batch_get_user_variable_units(post_processing_variables, self)
+
+        if not name_units_pair:
+            return
+
+        # Calculate column widths dynamically
+        name_column_width = max(len("Variable Name"), max(len(name) for name in name_units_pair))
+        unit_column_width = max(
+            len("Unit"), max(len(str(unit)) for unit in name_units_pair.values())
+        )
+
+        # Ensure minimum column widths
+        name_column_width = max(name_column_width, 15)
+        unit_column_width = max(unit_column_width, 10)
+
+        # Create the table header
+        header = f"{'Variable Name':<{name_column_width}} | {'Unit':<{unit_column_width}}"
+        separator = "-" * len(header)
+
+        # Print the table
+        log.info("")
+        log.info("Units of output `UserVariables`:")
+        log.info(separator)
+        log.info(header)
+        log.info(separator)
+
+        # Print each row
+        for name, unit in name_units_pair.items():
+            log.info(f"{name:<{name_column_width}} | {str(unit):<{unit_column_width}}")
+
+        log.info(separator)
+        log.info("")
+
+    def pre_submit_summary(self):
+        """
+        Display a summary of the simulation params before submission.
+        """
+        self.display_output_units()
