@@ -3,6 +3,12 @@
 # pylint: disable=too-many-lines
 from typing import Type, Union
 
+import unyt as u
+
+from flow360.component.simulation.conversion import (
+    LIQUID_IMAGINARY_FREESTREAM_MACH,
+    compute_udf_dimensionalization_factor,
+)
 from flow360.component.simulation.framework.entity_base import EntityList
 from flow360.component.simulation.models.material import Sutherland
 from flow360.component.simulation.models.solver_numerics import NoneSolver
@@ -28,6 +34,7 @@ from flow360.component.simulation.models.surface_models import (
 )
 from flow360.component.simulation.models.volume_models import (
     ActuatorDisk,
+    AngularVelocity,
     BETDisk,
     Fluid,
     NavierStokesInitialCondition,
@@ -36,8 +43,19 @@ from flow360.component.simulation.models.volume_models import (
     Rotation,
     Solid,
 )
-from flow360.component.simulation.outputs.output_entities import Point, PointArray
-from flow360.component.simulation.outputs.output_fields import generate_predefined_udf
+from flow360.component.simulation.operating_condition.operating_condition import (
+    LiquidOperatingCondition,
+)
+from flow360.component.simulation.outputs.output_entities import (
+    Point,
+    PointArray,
+    PointArray2D,
+)
+from flow360.component.simulation.outputs.output_fields import (
+    PREDEFINED_UDF_EXPRESSIONS,
+    append_component_to_output_fields,
+    generate_predefined_udf,
+)
 from flow360.component.simulation.outputs.outputs import (
     AeroAcousticOutput,
     Isosurface,
@@ -45,10 +63,12 @@ from flow360.component.simulation.outputs.outputs import (
     ProbeOutput,
     Slice,
     SliceOutput,
+    StreamlineOutput,
     SurfaceIntegralOutput,
     SurfaceOutput,
     SurfaceProbeOutput,
     SurfaceSliceOutput,
+    TimeAverageIsosurfaceOutput,
     TimeAverageProbeOutput,
     TimeAverageSliceOutput,
     TimeAverageSurfaceOutput,
@@ -60,18 +80,33 @@ from flow360.component.simulation.outputs.outputs import (
 from flow360.component.simulation.primitives import Box, SurfacePair
 from flow360.component.simulation.simulation_params import SimulationParams
 from flow360.component.simulation.time_stepping.time_stepping import Steady, Unsteady
+from flow360.component.simulation.translator.user_expression_utils import (
+    udf_prepending_code,
+)
 from flow360.component.simulation.translator.utils import (
     _get_key_name,
     convert_tuples_to_lists,
     get_global_setting_from_first_instance,
     has_instance_in_list,
+    inline_expressions_in_dict,
     preprocess_input,
     remove_units_in_dict,
     replace_dict_key,
     translate_setting_and_apply_to_all_entities,
+    translate_value_or_expression_object,
     update_dict_recursively,
 )
 from flow360.component.simulation.unit_system import LengthType
+from flow360.component.simulation.user_code.core.types import (
+    Expression,
+    UserVariable,
+    _convert_numeric,
+)
+from flow360.component.simulation.user_code.functions import math
+from flow360.component.simulation.user_code.variables import solution
+from flow360.component.simulation.user_defined_dynamics.user_defined_dynamics import (
+    UserDefinedDynamic,
+)
 from flow360.component.simulation.utils import (
     is_exact_instance,
     is_instance_of_type_in_union,
@@ -187,7 +222,7 @@ def rotation_entity_info_serializer(volume):
     }
 
 
-def rotation_translator(model: Rotation):
+def rotation_translator(model: Rotation, params: SimulationParams):
     """Rotation translator"""
     volume_zone = {
         "modelType": "FluidDynamics",
@@ -206,6 +241,10 @@ def rotation_translator(model: Rotation):
             and spec_value.get("units", "") == "flow360_angular_velocity_unit"
         ):
             volume_zone["referenceFrame"]["omegaRadians"] = spec_value["value"]
+        elif isinstance(model.spec, AngularVelocity):
+            volume_zone["referenceFrame"]["omegaRadians"] = translate_value_or_expression_object(
+                model.spec.value, params
+            )
     return volume_zone
 
 
@@ -219,14 +258,26 @@ def translate_output_fields(
         SurfaceIntegralOutput,
         SurfaceProbeOutput,
         SurfaceSliceOutput,
+        StreamlineOutput,
     ],
 ):
     """Get output fields"""
-    return {"outputFields": output_model.output_fields.items}
+    output_fields = []
+
+    for item in append_component_to_output_fields(output_model.output_fields.items):
+        output_fields.append(item)
+
+    for output_field in output_model.output_fields.items:
+        if isinstance(output_field, UserVariable):
+            # Remove the UserVariable object and add its name
+            output_fields.append(output_field.name)
+    # Filter out the UserVariable Dicts
+    output_fields = [item for item in output_fields if isinstance(item, str)]
+    return {"outputFields": output_fields}
 
 
 def surface_probe_setting_translation_func(entity: SurfaceProbeOutput):
-    """Translate non-entitties part of SurfaceProbeOutput"""
+    """Translate non-entities part of SurfaceProbeOutput"""
     dict_with_merged_output_fields = monitor_translator(entity)
     dict_with_merged_output_fields["surfacePatches"] = [
         surface.full_name for surface in entity.target_surfaces.stored_entities
@@ -269,11 +320,23 @@ def inject_surface_slice_info(entity: Slice):
     }
 
 
-def inject_isosurface_info(entity: Isosurface):
+def inject_isosurface_info(entity: Isosurface, input_params: SimulationParams):
     """inject entity info"""
+
+    if isinstance(entity.field, UserVariable):
+        units = entity.field.value.get_output_units(input_params=input_params)
+        surface_field = entity.field.name
+        surface_magnitude = (
+            entity.iso_value.to(units).v.item()
+            if not isinstance(entity.iso_value, float)
+            else entity.iso_value
+        )
+    else:
+        surface_field = entity.field
+        surface_magnitude = entity.iso_value
     return {
-        "surfaceField": entity.field,
-        "surfaceFieldMagnitude": entity.iso_value,
+        "surfaceField": surface_field,
+        "surfaceFieldMagnitude": surface_magnitude,
     }
 
 
@@ -340,11 +403,24 @@ def translate_volume_output(
         is_average=volume_output_class is TimeAverageVolumeOutput,
     )
     # Get outputFields
+    output_fields = []
+
+    output_fields = append_component_to_output_fields(
+        get_global_setting_from_first_instance(
+            output_params, volume_output_class, "output_fields"
+        ).model_dump()["items"]
+    )
+
+    for output_field in get_global_setting_from_first_instance(
+        output_params, volume_output_class, "output_fields"
+    ).items:
+        if isinstance(output_field, UserVariable):
+            output_fields.append(output_field.name)
+    # Filter out the UserVariable Dicts
+    output_fields = [item for item in output_fields if isinstance(item, str)]
     volume_output.update(
         {
-            "outputFields": get_global_setting_from_first_instance(
-                output_params, volume_output_class, "output_fields"
-            ).model_dump()["items"],
+            "outputFields": output_fields,
         }
     )
     return volume_output
@@ -402,6 +478,7 @@ def translate_slice_output(
 
 
 def translate_isosurface_output(
+    input_params: SimulationParams,
     output_params: list,
     injection_function,
 ):
@@ -418,6 +495,30 @@ def translate_isosurface_output(
         translation_func=translate_output_fields,
         to_list=False,
         entity_injection_func=injection_function,
+        entity_injection_input_params=input_params,
+    )
+    return translated_output
+
+
+def translate_time_average_isosurface_output(
+    input_params: SimulationParams,
+    output_params: list,
+    injection_function,
+):
+    """Translate time average isosurface output settings."""
+    translated_output = init_output_base(
+        output_params,
+        TimeAverageIsosurfaceOutput,
+        has_average_capability=True,
+        is_average=True,
+    )
+    translated_output["isoSurfaces"] = translate_setting_and_apply_to_all_entities(
+        output_params,
+        TimeAverageIsosurfaceOutput,
+        translation_func=translate_output_fields,
+        to_list=False,
+        entity_injection_func=injection_function,
+        entity_injection_input_params=input_params,
     )
     return translated_output
 
@@ -495,7 +596,81 @@ def translate_acoustic_output(output_params: list):
     return None
 
 
-def process_output_fields_for_udf(input_params):
+def user_variable_to_udf(
+    variable: UserVariable, input_params: SimulationParams
+):  # pylint:disable=too-many-branches
+    # pylint:disable=too-many-statements
+    """Convert user variable to UDF"""
+    if not isinstance(variable.value, Expression):
+        expression = Expression.model_validate(_convert_numeric(variable.value))
+    else:
+        expression = variable.value
+
+    def _prepare_prepending_code(expression: Expression):
+        prepending_code = []
+        for name in sorted(expression.solver_variable_names(recursive=True)):
+            if not udf_prepending_code.get(name):
+                continue
+            if name == "solution.temperature" and input_params.has_solid():
+                prepending_code.append(udf_prepending_code["solution.temperature_solid"])
+                continue
+            prepending_code.append(udf_prepending_code[name])
+        prepending_code = "".join(prepending_code)
+        return prepending_code
+
+    requested_unit: Union[u.Unit, None] = expression.get_output_units(input_params=input_params)
+    if requested_unit is None:
+        # Number constant output requested
+        coefficient = 1
+        offset = 0
+    else:
+        coefficient, offset = compute_udf_dimensionalization_factor(
+            params=input_params,
+            requested_unit=requested_unit,
+            using_liquid_op=isinstance(input_params.operating_condition, LiquidOperatingCondition),
+        )
+
+    expression_length = expression.length
+    prepending_code = _prepare_prepending_code(expression=expression)
+    if expression_length == 0:  # Scalar output requested
+        expression = expression.evaluate(raise_on_non_evaluable=False, force_evaluate=False)
+        if offset != 0:
+            expression = (expression + offset) * coefficient
+        else:
+            expression = expression * coefficient
+        if not isinstance(expression, Expression):
+            # Enforce constant as Expression
+            expression = Expression.model_validate(_convert_numeric(expression))
+        expression = expression.to_solver_code(params=input_params)
+        return UserDefinedField(
+            name=variable.name, expression=f"{prepending_code}{variable.name} = " + expression + ";"
+        )
+
+    # Vector output requested
+    expression = [
+        expression[i].evaluate(raise_on_non_evaluable=False, force_evaluate=False)
+        for i in range(expression_length)
+    ]
+    if offset != 0:
+        expression = [(item + offset) * coefficient for item in expression]
+    else:
+        expression = [item * coefficient for item in expression]
+    for i, item in enumerate(expression):
+        if isinstance(item, Expression):
+            expression[i] = item.to_solver_code(params=input_params)
+        else:
+            expression[i] = Expression.model_validate(_convert_numeric(item)).to_solver_code(
+                params=input_params
+            )
+
+    expression = [f"{variable.name}[{i}] = " + item for i, item in enumerate(expression)]
+    return UserDefinedField(
+        name=variable.name, expression=prepending_code + "; ".join(expression) + ";"
+    )
+
+
+def process_output_fields_for_udf(input_params: SimulationParams):
+    # pylint:disable=too-many-branches
     """
     Process all output fields from different output types and generate additional
     UserDefinedFields for dimensioned fields.
@@ -515,7 +690,16 @@ def process_output_fields_for_udf(input_params):
     if input_params.outputs:
         for output in input_params.outputs:
             if hasattr(output, "output_fields") and output.output_fields:
-                all_field_names.update(output.output_fields.items)
+                all_field_names.update(
+                    append_component_to_output_fields(output.output_fields.items)
+                )
+            if isinstance(output, IsosurfaceOutput):
+                for isosurface in output.entities.items:
+                    if isosurface.field in PREDEFINED_UDF_EXPRESSIONS:
+                        all_field_names.add(isosurface.field)
+
+    if isinstance(input_params.operating_condition, LiquidOperatingCondition):
+        all_field_names.add("velocity_magnitude")
 
     # Generate UDFs for dimensioned fields
     generated_udfs = []
@@ -524,11 +708,58 @@ def process_output_fields_for_udf(input_params):
         if udf_expression:
             generated_udfs.append(UserDefinedField(name=field_name, expression=udf_expression))
 
-    return generated_udfs
+    # UserVariable handling:
+    user_variable_udfs = {}
+    if input_params.outputs:
+        for output in input_params.outputs:
+            if hasattr(output, "output_fields") and output.output_fields:
+                for output_field in output.output_fields.items:
+                    if not isinstance(output_field, UserVariable):
+                        continue
+                    udf_from_user_variable = user_variable_to_udf(output_field, input_params)
+                    user_variable_udfs[udf_from_user_variable.name] = udf_from_user_variable
+            if isinstance(output, IsosurfaceOutput):
+                for isosurface in output.entities.items:
+                    if not isinstance(isosurface.field, UserVariable):
+                        continue
+                    udf_from_user_variable = user_variable_to_udf(isosurface.field, input_params)
+                    user_variable_udfs[udf_from_user_variable.name] = udf_from_user_variable
+    return generated_udfs, list(user_variable_udfs.values())
+
+
+def translate_streamline_output(output_params: list):
+    """Translate streamline output settings."""
+    streamline_output = {"Points": [], "PointArrays": [], "PointArrays2D": []}
+    for output in output_params:
+        if isinstance(output, StreamlineOutput):
+            for entity in output.entities.stored_entities:
+                if isinstance(entity, Point):
+                    point = {"name": entity.name, "location": entity.location.value.tolist()}
+                    streamline_output["Points"].append(point)
+                elif isinstance(entity, PointArray):
+                    line = {
+                        "name": entity.name,
+                        "start": entity.start.value.tolist(),
+                        "end": entity.end.value.tolist(),
+                        "numberOfPoints": entity.number_of_points,
+                    }
+                    streamline_output["PointArrays"].append(line)
+                elif isinstance(entity, PointArray2D):
+                    parallelogram = {
+                        "name": entity.name,
+                        "origin": entity.origin.value.tolist(),
+                        "uAxisVector": entity.u_axis_vector.value.tolist(),
+                        "vAxisVector": entity.v_axis_vector.value.tolist(),
+                        "uNumberOfPoints": entity.u_number_of_points,
+                        "vNumberOfPoints": entity.v_number_of_points,
+                    }
+                    streamline_output["PointArrays2D"].append(parallelogram)
+
+    return streamline_output
 
 
 def translate_output(input_params: SimulationParams, translated: dict):
-    # pylint: disable=too-many-branches,too-many-statements
+    # pylint: disable=too-many-branches,too-many-statements,too-many-locals
     """Translate output settings."""
     outputs = input_params.outputs
 
@@ -577,7 +808,11 @@ def translate_output(input_params: SimulationParams, translated: dict):
     ##:: Step4: Get translated["isoSurfaceOutput"]
     if has_instance_in_list(outputs, IsosurfaceOutput):
         translated["isoSurfaceOutput"] = translate_isosurface_output(
-            outputs, inject_isosurface_info
+            input_params, outputs, inject_isosurface_info
+        )
+    if has_instance_in_list(outputs, TimeAverageIsosurfaceOutput):
+        translated["timeAverageIsoSurfaceOutput"] = translate_time_average_isosurface_output(
+            input_params, outputs, inject_isosurface_info
         )
 
     ##:: Step5: Get translated["monitorOutput"]
@@ -591,6 +826,33 @@ def translate_output(input_params: SimulationParams, translated: dict):
             outputs, TimeAverageProbeOutput, inject_probe_info
         )
     if has_instance_in_list(outputs, SurfaceIntegralOutput):
+        for output in outputs:
+            if not isinstance(output, SurfaceIntegralOutput):
+                continue
+            output_fields_processed = []
+            for output_field in output.output_fields.items:
+                if isinstance(output_field, UserVariable):
+                    expression = Expression.model_validate(output_field.value)
+                    if expression.length == 0:
+                        expression_processed = expression * math.magnitude(
+                            solution.node_area_vector
+                        )
+                    else:
+                        expression_processed = [
+                            expression[i] * math.magnitude(solution.node_area_vector)
+                            for i in range(expression.length)
+                        ]
+
+                    output_fields_processed.append(
+                        UserVariable(
+                            name=output_field.name + "_integral",
+                            value=expression_processed,
+                        )
+                    )
+                else:
+                    output_fields_processed.append(output_field)
+            output.output_fields.items = output_fields_processed
+
         integral_output = translate_monitor_output(
             outputs, SurfaceIntegralOutput, inject_surface_list_info
         )
@@ -632,6 +894,24 @@ def translate_output(input_params: SimulationParams, translated: dict):
     if has_instance_in_list(outputs, AeroAcousticOutput):
         translated["aeroacousticOutput"] = translate_acoustic_output(outputs)
 
+    ##:: Step7: Get translated["streamlineOutput"]
+    if has_instance_in_list(outputs, StreamlineOutput):
+        translated["streamlineOutput"] = translate_streamline_output(outputs)
+
+    ##:: Step8: Sort all "output_fields" everywhere
+    # Recursively sort all "outputFields" lists in the translated dict
+    def _sort_output_fields_in_dict(d):
+        if isinstance(d, dict):
+            for k, v in d.items():
+                if k == "outputFields" and isinstance(v, list):
+                    v.sort()
+                else:
+                    _sort_output_fields_in_dict(v)
+        elif isinstance(d, list):
+            for item in d:
+                _sort_output_fields_in_dict(item)
+
+    _sort_output_fields_in_dict(translated)
     return translated
 
 
@@ -798,6 +1078,87 @@ def _append_turbulence_quantities_to_dict(model, model_dict, boundary):
     return boundary
 
 
+def _get_default_mass_outflow_udd(entities, mass_flow_rate):
+    """Default mass outflow UDD translator"""
+    proportional_coefficient = 1.0e-2
+    udd_list = []
+    for entity in entities.stored_entities:
+        udd = UserDefinedDynamic(
+            name=f"massOutflowController_{entity.name}",
+            input_vars=["massFlowRate", "area", "hasSupersonicFlow"],
+            constants={
+                "massFlowRateTarget": mass_flow_rate,
+                "Kp": proportional_coefficient,
+                "initialStaticPressureRatio": 1.0,
+            },
+            output_vars={
+                "staticPressureRatio": "if (pseudoStep > 0) state[0]; else initialStaticPressureRatio;"
+            },
+            state_vars_initial_value=["initialStaticPressureRatio"],
+            update_law=[
+                "if (hasSupersonicFlow and massFlowRate > 0) (1.0 + Kp) * state[0]; else state[0] + "
+                + "Kp * (massFlowRate/area - massFlowRateTarget/area);"
+            ],
+            input_boundary_patches=entities.stored_entities,
+            output_target=entity,
+        )
+        udd_list.append(udd)
+
+    return udd_list
+
+
+def _get_default_mass_inflow_udd(entities, mass_flow_rate):
+    """Default mass inflow UDD translator"""
+    proportional_coefficient = 1.0e-2
+    udd_list = []
+    for entity in entities.stored_entities:
+        udd = UserDefinedDynamic(
+            name=f"massInflowController_{entity.name}",
+            input_vars=["massFlowRate", "area", "hasSupersonicFlow"],
+            constants={
+                "massFlowRateTarget": mass_flow_rate,
+                "Kp": proportional_coefficient,
+                "initialTotalPressureRatio": 1.0,
+            },
+            output_vars={
+                "totalPressureRatio": "if (pseudoStep > 0) state[0]; else initialTotalPressureRatio;"
+            },
+            state_vars_initial_value=["initialTotalPressureRatio"],
+            update_law=[
+                "if (hasSupersonicFlow and massFlowRate > 0) (1.0 - Kp) * state[0]; else state[0] - "
+                + "Kp * (massFlowRate/area - massFlowRateTarget/area);"
+            ],
+            input_boundary_patches=entities.stored_entities,
+            output_target=entity,
+        )
+        udd_list.append(udd)
+
+    return udd_list
+
+
+def mass_flow_default_udd(models, user_defined_dynamics):
+    """Default mass flow rate translator"""
+    mass_flow_user_defined_dynamics = []
+    for model in models:
+        if isinstance(model, Outflow):
+            if isinstance(model.spec, MassFlowRate):
+                udd = _get_default_mass_outflow_udd(model.entities, model.spec.value)
+                mass_flow_user_defined_dynamics.extend(udd)
+        elif isinstance(model, Inflow):
+            if isinstance(model.spec, MassFlowRate):
+                udd = _get_default_mass_inflow_udd(model.entities, model.spec.value)
+                mass_flow_user_defined_dynamics.extend(udd)
+
+    if len(mass_flow_user_defined_dynamics) == 0:
+        return user_defined_dynamics
+
+    if user_defined_dynamics is None:
+        user_defined_dynamics = []
+
+    user_defined_dynamics.extend(mass_flow_user_defined_dynamics)
+    return user_defined_dynamics
+
+
 # pylint: disable=too-many-branches, too-many-statements
 def boundary_spec_translator(model: SurfaceModelTypes, op_acoustic_to_static_pressure_ratio):
     """Boundary translator"""
@@ -837,15 +1198,17 @@ def boundary_spec_translator(model: SurfaceModelTypes, op_acoustic_to_static_pre
         elif isinstance(model.heat_spec, HeatFlux):
             boundary["heatFlux"] = model_dict["heatSpec"]["value"]
         boundary["roughnessHeight"] = model_dict["roughnessHeight"]
+        if model.private_attribute_dict is not None:
+            boundary = {**boundary, **model.private_attribute_dict}
     elif isinstance(model, Inflow):
         boundary["totalTemperatureRatio"] = model_dict["totalTemperature"]
+        if model.velocity_direction is not None:
+            boundary["velocityDirection"] = list(model_dict["velocityDirection"])
         if isinstance(model.spec, TotalPressure):
             boundary["type"] = "SubsonicInflow"
             boundary["totalPressureRatio"] = (
                 model_dict["spec"]["value"] * op_acoustic_to_static_pressure_ratio
             )
-            if model.spec.velocity_direction is not None:
-                boundary["velocityDirection"] = list(model_dict["spec"]["velocityDirection"])
         elif isinstance(model.spec, MassFlowRate):
             boundary["type"] = "MassInflow"
             boundary["massFlowRate"] = model_dict["spec"]["value"]
@@ -886,7 +1249,7 @@ def boundary_spec_translator(model: SurfaceModelTypes, op_acoustic_to_static_pre
 
 
 def get_navier_stokes_initial_condition(
-    initial_condition: Union[NavierStokesInitialCondition, NavierStokesModifiedRestartSolution]
+    initial_condition: Union[NavierStokesInitialCondition, NavierStokesModifiedRestartSolution],
 ):
     """Translate the initial condition for NavierStokes"""
     if is_exact_instance(initial_condition, NavierStokesInitialCondition):
@@ -911,6 +1274,49 @@ def get_navier_stokes_initial_condition(
     return initial_condition_dict
 
 
+def rename_modeling_constants(modeling_constants):
+    """Rename the modeling constants to what the solver reads"""
+    if modeling_constants.get("typeName", None) == "SpalartAllmarasConsts":
+        replace_dict_key(modeling_constants, "CDES", "C_DES")
+        replace_dict_key(modeling_constants, "CD", "C_d")
+        replace_dict_key(modeling_constants, "CCb1", "C_cb1")
+        replace_dict_key(modeling_constants, "CCb2", "C_cb2")
+        replace_dict_key(modeling_constants, "CSigma", "C_sigma")
+        replace_dict_key(modeling_constants, "CV1", "C_v1")
+        replace_dict_key(modeling_constants, "CVonKarman", "C_vonKarman")
+        replace_dict_key(modeling_constants, "CW2", "C_w2")
+        replace_dict_key(modeling_constants, "CT3", "C_t3")
+        replace_dict_key(modeling_constants, "CT4", "C_t4")
+        replace_dict_key(modeling_constants, "CMinRd", "C_min_rd")
+
+    if modeling_constants.get("typeName", None) == "kOmegaSSTConsts":
+        replace_dict_key(modeling_constants, "CDES1", "C_DES1")
+        replace_dict_key(modeling_constants, "CDES2", "C_DES2")
+        replace_dict_key(modeling_constants, "CD1", "C_d1")
+        replace_dict_key(modeling_constants, "CD2", "C_d2")
+        replace_dict_key(modeling_constants, "CSigmaK1", "C_sigma_k1")
+        replace_dict_key(modeling_constants, "CSigmaK2", "C_sigma_k2")
+        replace_dict_key(modeling_constants, "CSigmaOmega1", "C_sigma_omega1")
+        replace_dict_key(modeling_constants, "CSigmaOmega2", "C_sigma_omega2")
+        replace_dict_key(modeling_constants, "CAlpha1", "C_alpha1")
+        replace_dict_key(modeling_constants, "CBeta1", "C_beta1")
+        replace_dict_key(modeling_constants, "CBeta2", "C_beta2")
+        replace_dict_key(modeling_constants, "CBetaStar", "C_beta_star")
+
+    modeling_constants.pop("typeName")  # Not read by solver
+
+
+def update_controls_modeling_constants(controls, translated):
+    """Upading the modelingConstants entries for each control"""
+    if controls is not None:
+        for control in translated["turbulenceModelSolver"]["controls"]:
+            control_modeling_constants = control.get("modelingConstants", None)
+            if control_modeling_constants is None:
+                continue
+            rename_modeling_constants(control_modeling_constants)
+            control["modelConstants"] = control.pop("modelingConstants")
+
+
 # pylint: disable=too-many-statements
 # pylint: disable=too-many-branches
 # pylint: disable=too-many-locals
@@ -927,7 +1333,10 @@ def get_solver_json(
     translated = {}
     ##:: Step 1: Get geometry:
     if input_params.reference_geometry:
-        geometry = remove_units_in_dict(dump_dict(input_params.reference_geometry))
+        geometry = inline_expressions_in_dict(
+            dump_dict(input_params.reference_geometry), input_params
+        )
+        geometry = remove_units_in_dict(geometry, skip_keys=["private_attribute_area_settings"])
         translated["geometry"] = {}
         if input_params.reference_geometry.area is not None:
             translated["geometry"]["refArea"] = geometry["area"]
@@ -944,21 +1353,35 @@ def get_solver_json(
     # check if all units are flow360:
     _ = remove_units_in_dict(dump_dict(op))
     translated["freestream"] = {
-        "alphaAngle": op.alpha.to("degree").v.item() if "alpha" in op.model_fields else 0,
-        "betaAngle": op.beta.to("degree").v.item() if "beta" in op.model_fields else 0,
-        "Mach": op.velocity_magnitude.v.item(),
+        "alphaAngle": op.alpha.to("degree").v.item() if "alpha" in op.__class__.model_fields else 0,
+        "betaAngle": op.beta.to("degree").v.item() if "beta" in op.__class__.model_fields else 0,
+        "Mach": translate_value_or_expression_object(op.velocity_magnitude, input_params),
         "Temperature": (
             op.thermal_state.temperature.to("K").v.item()
-            if isinstance(op.thermal_state.material.dynamic_viscosity, Sutherland)
+            if not isinstance(op, LiquidOperatingCondition)
+            and isinstance(op.thermal_state.material.dynamic_viscosity, Sutherland)
             else -1
         ),
-        "muRef": op.thermal_state.dynamic_viscosity.v.item(),
+        "muRef": (
+            op.thermal_state.dynamic_viscosity.v.item()
+            if not isinstance(op, LiquidOperatingCondition)
+            else op.material.dynamic_viscosity.v.item()
+        ),
     }
-    if "reference_velocity_magnitude" in op.model_fields.keys() and op.reference_velocity_magnitude:
+    if (
+        "reference_velocity_magnitude" in op.__class__.model_fields.keys()
+        and op.reference_velocity_magnitude
+    ):
         translated["freestream"]["MachRef"] = op.reference_velocity_magnitude.v.item()
     op_acoustic_to_static_pressure_ratio = (
-        op.thermal_state.density * op.thermal_state.speed_of_sound**2 / op.thermal_state.pressure
-    ).value
+        (
+            op.thermal_state.density
+            * op.thermal_state.speed_of_sound**2
+            / op.thermal_state.pressure
+        ).value
+        if not isinstance(op, LiquidOperatingCondition)
+        else 1.0
+    )
 
     ##:: Step 5: Get timeStepping
     ts = input_params.time_stepping
@@ -968,7 +1391,7 @@ def get_solver_json(
             "physicalSteps": ts.steps,
             "orderOfAccuracy": ts.order_of_accuracy,
             "maxPseudoSteps": ts.max_pseudo_steps,
-            "timeStepSize": ts.step_size.value.item(),
+            "timeStepSize": translate_value_or_expression_object(ts.step_size, input_params),
         }
     elif isinstance(ts, Steady):
         translated["timeStepping"] = {
@@ -983,12 +1406,20 @@ def get_solver_json(
     ##:: Step 6: Get solver settings and initial condition
     for model in input_params.models:
         if isinstance(model, Fluid):
-            if model.navier_stokes_solver.low_mach_preconditioner:
-                if model.navier_stokes_solver.low_mach_preconditioner_threshold is None:
-                    model.navier_stokes_solver.low_mach_preconditioner_threshold = (
-                        input_params.operating_condition.mach
-                    )
+            if isinstance(op, LiquidOperatingCondition):
+                model.navier_stokes_solver.low_mach_preconditioner = True
+                model.navier_stokes_solver.low_mach_preconditioner_threshold = (
+                    LIQUID_IMAGINARY_FREESTREAM_MACH
+                )
+            if (
+                model.navier_stokes_solver.low_mach_preconditioner
+                and model.navier_stokes_solver.low_mach_preconditioner_threshold is None
+            ):
+                model.navier_stokes_solver.low_mach_preconditioner_threshold = (
+                    input_params.operating_condition.mach
+                )
             translated["navierStokesSolver"] = dump_dict(model.navier_stokes_solver)
+
             replace_dict_key(translated["navierStokesSolver"], "typeName", "modelType")
             replace_dict_key(
                 translated["navierStokesSolver"],
@@ -1005,34 +1436,7 @@ def get_solver_json(
             replace_dict_key(translated["turbulenceModelSolver"], "typeName", "modelType")
             modeling_constants = translated["turbulenceModelSolver"].get("modelingConstants", None)
             if modeling_constants is not None:
-                if modeling_constants.get("typeName", None) == "SpalartAllmarasConsts":
-                    replace_dict_key(modeling_constants, "CDES", "C_DES")
-                    replace_dict_key(modeling_constants, "CD", "C_d")
-                    replace_dict_key(modeling_constants, "CCb1", "C_cb1")
-                    replace_dict_key(modeling_constants, "CCb2", "C_cb2")
-                    replace_dict_key(modeling_constants, "CSigma", "C_sigma")
-                    replace_dict_key(modeling_constants, "CV1", "C_v1")
-                    replace_dict_key(modeling_constants, "CVonKarman", "C_vonKarman")
-                    replace_dict_key(modeling_constants, "CW2", "C_w2")
-                    replace_dict_key(modeling_constants, "CT3", "C_t3")
-                    replace_dict_key(modeling_constants, "CT4", "C_t4")
-                    replace_dict_key(modeling_constants, "CMinRd", "C_min_rd")
-
-                if modeling_constants.get("typeName", None) == "kOmegaSSTConsts":
-                    replace_dict_key(modeling_constants, "CDES1", "C_DES1")
-                    replace_dict_key(modeling_constants, "CDES2", "C_DES2")
-                    replace_dict_key(modeling_constants, "CD1", "C_d1")
-                    replace_dict_key(modeling_constants, "CD2", "C_d2")
-                    replace_dict_key(modeling_constants, "CSigmaK1", "C_sigma_k1")
-                    replace_dict_key(modeling_constants, "CSigmaK2", "C_sigma_k2")
-                    replace_dict_key(modeling_constants, "CSigmaOmega1", "C_sigma_omega1")
-                    replace_dict_key(modeling_constants, "CSigmaOmega2", "C_sigma_omega2")
-                    replace_dict_key(modeling_constants, "CAlpha1", "C_alpha1")
-                    replace_dict_key(modeling_constants, "CBeta1", "C_beta1")
-                    replace_dict_key(modeling_constants, "CBeta2", "C_beta2")
-                    replace_dict_key(modeling_constants, "CBetaStar", "C_beta_star")
-
-                modeling_constants.pop("typeName")  # Not read by solver
+                rename_modeling_constants(modeling_constants)
                 translated["turbulenceModelSolver"]["modelConstants"] = translated[
                     "turbulenceModelSolver"
                 ].pop("modelingConstants")
@@ -1055,6 +1459,10 @@ def get_solver_json(
                     translated["turbulenceModelSolver"]["ZDES"] = False
                     translated["turbulenceModelSolver"]["gridSizeForLES"] = "maxEdgeLength"
 
+                update_controls_modeling_constants(
+                    model.turbulence_model_solver.controls, translated
+                )
+
             if not isinstance(model.transition_model_solver, NoneSolver):
                 # baseline dictionary dump for transition model object
                 translated["transitionModelSolver"] = dump_dict(model.transition_model_solver)
@@ -1112,6 +1520,7 @@ def get_solver_json(
                 rotation_translator,
                 to_list=False,
                 entity_injection_func=rotation_entity_info_serializer,
+                translation_func_params=input_params,
             )
         )
         translated["volumeZones"] = volume_zones
@@ -1222,17 +1631,28 @@ def get_solver_json(
 
     ##:: Step 5: Get user defined fields and auto-generated fields for dimensioned output
     translated["userDefinedFields"] = []
-    # Add auto-generated UDFs for dimensioned fields
-    generated_udfs = process_output_fields_for_udf(input_params)
+    # Add auto-generated UDFs for dimensioned fields + user variable UDFs
+    generated_udfs, user_variable_udfs = process_output_fields_for_udf(input_params)
 
     # Add user-specified UDFs and auto-generated UDFs for dimensioned fields
-    for udf in [*input_params.user_defined_fields, *generated_udfs]:
+    legacy_udf_count = len(input_params.user_defined_fields) + len(generated_udfs)
+    for udf_count, udf in enumerate(
+        [*input_params.user_defined_fields, *generated_udfs, *user_variable_udfs]
+    ):
         udf_dict = {}
         udf_dict["name"] = udf.name
         udf_dict["expression"] = udf.expression
+        if udf_count < legacy_udf_count:
+            udf_dict["from_user_variables"] = False
         translated["userDefinedFields"].append(udf_dict)
 
+    translated["userDefinedFields"].sort(key=lambda udf: udf["name"])
+
     ##:: Step 11: Get user defined dynamics
+    input_params.user_defined_dynamics = mass_flow_default_udd(
+        input_params.models, input_params.user_defined_dynamics
+    )
+
     if input_params.user_defined_dynamics is not None:
         translated["userDefinedDynamics"] = []
         for udd in input_params.user_defined_dynamics:
@@ -1240,6 +1660,7 @@ def get_solver_json(
             udd_dict_translated = {}
             udd_dict_translated["dynamicsName"] = udd_dict.pop("name")
             udd_dict_translated["inputVars"] = udd_dict.pop("inputVars", [])
+            udd_dict_translated["inputVars"].sort()
             udd_dict_translated["outputVars"] = udd_dict.pop("outputVars", [])
             udd_dict_translated["stateVarsInitialValue"] = udd_dict.pop("stateVarsInitialValue", [])
             udd_dict_translated["updateLaw"] = udd_dict.pop("updateLaw", [])
@@ -1248,8 +1669,24 @@ def get_solver_json(
                 udd_dict_translated["inputBoundaryPatches"] = []
                 for surface in udd.input_boundary_patches.stored_entities:
                     udd_dict_translated["inputBoundaryPatches"].append(_get_key_name(surface))
+                udd_dict_translated["inputBoundaryPatches"].sort()
             if udd.output_target is not None:
                 udd_dict_translated["outputTargetName"] = udd.output_target.full_name
             translated["userDefinedDynamics"].append(udd_dict_translated)
 
+        translated["userDefinedDynamics"].sort(key=lambda udd: udd["dynamicsName"])
+
+    translated["usingLiquidAsMaterial"] = isinstance(
+        input_params.operating_condition, LiquidOperatingCondition
+    )
+    translated["outputRescale"] = {"velocityScale": 1.0}
+    if isinstance(input_params.operating_condition, LiquidOperatingCondition):
+        translated["outputRescale"]["velocityScale"] = (
+            1.0 / translated["freestream"]["MachRef"]
+            if "MachRef" in translated["freestream"].keys()
+            else 1.0 / translated["freestream"]["Mach"]
+        )
+    if input_params.private_attribute_dict is not None:
+        translated.update(input_params.private_attribute_dict)
+
     return translated
